/**
 * Copyright (C) 2021 fortiss GmbH
 * @author Erkan Karabulut – "karabulut@fortiss.org"
 * @version 1.2
 * Create/describe list, and delete service requests
 */

import React from 'react';
import {
    View,
    Text,
    SafeAreaView,
    TextInput,
    ToastAndroid,
    TouchableOpacity,
    ScrollView, Image,
} from 'react-native';
import {Picker} from '@react-native-picker/picker'
import {connect} from 'react-redux';
import EStyleSheet from 'react-native-extended-stylesheet';
import BottomNavigation from "../component/BottomNavigation";
import Constants from "../constant/constants";
import * as Colors from "../constant/string/Colors";
import MQTTClientService from "../service/MQTTClientService";
import uuid from 'react-native-uuid';
import store from "../config/store";
import Icon from "react-native-vector-icons/Entypo";
import AspectService from "../service/AspectService";
import MeasurementUnitService from "../service/MeasurementUnitService";
import _ from "underscore";
import DropDownPicker from 'react-native-dropdown-picker';
import ServiceRequestParser from "../service/ServiceRequestParser";
import CheckBox from '@react-native-community/checkbox';
import {Tooltip} from "react-native-elements";
import DocumentPicker, {
    DirectoryPickerResponse,
    DocumentPickerResponse,
    isInProgress,
    types,
} from 'react-native-document-picker'
import * as RNFS from "react-native-fs";


// This class includes UI and business logic of creating, listing and deleting service requests
class Describe extends React.Component {
    constructor(props) {
        super(props);
        this.state = {
            updateSchedule: this.props.messages.everyDay,
            areaOfInterest: this.props.messages.perRoom,
            threshold: "30",
            selectedMeasurementType: null,
            selectedMonitoringAspect: "places",
            selectedMonitoringAspectDetail: "environment",
            selectedDomain: "buildingManagement",
            selectedDomainDetail: "offices",
            label: this.props.messages.temperature,
            monitoredAspectList: [],
            isMounted: false,
            serviceRequestViaFile: false,
            pickedFileName: this.props.messages.fileNotSelected,
            pickedFileURI: null,
            responseList: [],
            requestQueue: [],
            fileName: "EnvironmentMonitoringService.owl",
            fileList: [
                {value: "EnvironmentMonitoringService.owl", label: "EnvironmentMonitoringService.owl"},
                {value: "MonitoringEnvironmentService_wsdl2.xml", label: "MonitoringEnvironmentService_wsdl2.xml"},
                {value: "NewService.xml", label: "NewService.xml"},
            ],
            serviceRequestHost: "10.0.5.213",
            serviceRequestPort: "8000",
            username: "username",
            password: "password"
        };

        // type of the aspects that can be monitored
        this.measurementTypes = new Map();
        this.measurementTypes.set("motion", this.props.messages.motion);
        this.measurementTypes.set("position", this.props.messages.position);
        this.measurementTypes.set("energy", this.props.messages.energy);
        this.measurementTypes.set("environment", this.props.messages.environment);
        this.measurementTypes.set("mass", this.props.messages.mass);
        this.measurementTypes.set("biosensor", this.props.messages.biosensor);

        // initiate domain types
        this.domainTypes = new Map();
        this.domainTypes.set("health", this.props.messages.health);
        this.domainTypes.set("logistics", this.props.messages.logistics);
        this.domainTypes.set("agriculture", this.props.messages.agriculture);
        this.domainTypes.set("smartCity", this.props.messages.smartCity);
        this.domainTypes.set("buildingManagement", this.props.messages.buildingManagement);
        this.domainTypes.set("manufacturing", this.props.messages.manufacturing);

        // initiate monitoring aspect types
        this.monitoringAspectSentenceTypes = new Map();
        this.monitoringAspectSentenceTypes.set("objectsThings", this.props.messages.objects + "/" + this.props.messages.things);
        this.monitoringAspectSentenceTypes.set("places", this.props.messages.places);
    }

    componentDidMount() {
        // update monitored aspect list
        this.setState({monitoredAspectList: this.getMonitoredAspectList()});

        // Get sensor map and sort them alphabetically
        let sortedSensors = _.sortBy(Array.from(MeasurementUnitService.getSensorMap(this.props.messages).values()), function (sensor) {
            return sensor;
        });

        let sensorList = [];
        sortedSensors.forEach((sensorType) => {
            sensorList.push({value: sensorType, label: sensorType});
        });

        this.setState({sensorList: sensorList});

        store.subscribe((event) => {
            this.setState({monitoredAspectList: this.getMonitoredAspectList()});
        });
    }

    // Create lists for update schedule and area of interests
    getAreaOfInterestList() {
        let areaOfInterest = [];
        areaOfInterest.push(this.props.messages.perDevice);
        areaOfInterest.push(this.props.messages.perRoom);
        areaOfInterest.push(this.props.messages.perFloor);
        areaOfInterest.push(this.props.messages.perBuilding);

        return areaOfInterest;
    }

    // Triggered when user submits a request
    submit = async () => {
        let requestList = [];
        if (this.state.serviceRequestViaFile) {
            if (this.state.pickedFileURI == null) {
                ToastAndroid.showWithGravityAndOffset(this.props.messages.fillAllPlacesError, ToastAndroid.LONG, ToastAndroid.TOP, 0, 0)
                return;
            }

            RNFS.readFile(this.state.pickedFileURI).then(async (data) => {
                requestList = await ServiceRequestParser.getRequestsFromFile({
                    content: data,
                    fileName: this.state.pickedFileName
                });
                if (!requestList) {
                    ToastAndroid.showWithGravityAndOffset(this.props.messages.invalidURL, ToastAndroid.LONG, ToastAndroid.TOP, 0, 0);
                    return;
                }

                this.setState({requestQueue: requestList});
            });
        } else {
            // check if all necessary fields are filled
            if (this.state.updateSchedule == null || this.state.areaOfInterest == null ||
                this.state.threshold == null || this.state.selectedMeasurementType == null) {
                ToastAndroid.showWithGravityAndOffset(this.props.messages.fillAllPlacesError, ToastAndroid.LONG, ToastAndroid.TOP, 0, 0);
                return;
            }

            // create a request object
            let manuallyCreatedRequest = {
                "QuantityKind": this.state.selectedMeasurementType,
                "featureOfInterest": this.state.selectedMeasurementType,
                "unitOfMeasurement": MeasurementUnitService.getUnitOfMeasurementByAspect(this.state.selectedMeasurementType),
                "threshold": this.state.threshold,
                "creatorId": Constants.constants.CREATOR_ID,
                "location": this.state.areaOfInterest.replace(" ", "-"),
                "uuid": uuid.v4(),
                "domain": this.domainTypes.get(this.state.selectedDomain),
                "domainDetail": MeasurementUnitService.getDomainMapByType(this.state.selectedDomain, this.props.messages)
                    .get(this.state.selectedDomainDetail),
                "monitoringAspect": this.monitoringAspectSentenceTypes.get(this.state.selectedMonitoringAspect),
                "monitoringAspectDetail": MeasurementUnitService.getMonitoringAspectMapByType(this.state.selectedMonitoringAspect,
                    this.props.messages).get(this.state.selectedMonitoringAspectDetail),
            }

            requestList.push(manuallyCreatedRequest);
            this.state.requestQueue.push(manuallyCreatedRequest);
        }

        if (MQTTClientService.mqtt.isConnected()) {
            for (let request of requestList) {
                request.isExternal = false;
                store.dispatch({type: "ADD_REQUEST", request: request});
                MQTTClientService.send(Constants.constants.OBSERVATION_REQUEST_TOPIC, JSON.stringify(request));
            }

            let request = this.state.requestQueue.pop();
            setTimeout(() => {
                this.waitForResponse(request, 0);
            }, 1000);
        } else {
            ToastAndroid.showWithGravityAndOffset(this.props.messages.notConnectedToBroker, ToastAndroid.LONG, ToastAndroid.TOP, 0, 0);
        }
    }

    /**
     * wait for a response for the next request
     */
    scheduleNextRequest() {
        if (this.state.requestQueue.length === 0) {
            let resultList = this.state.responseList;
            this.setState({responseList: []});

            for (let response of this.props.responseList) {
                if (response && response.name === "groupingNotPossible") {
                    store.dispatch({type: "REMOVE_RESPONSE", response: response});
                    store.dispatch({
                        type: "REMOVE_REQUEST",
                        request: AspectService.findRequestById(response.requestId, this.props.requestList)
                    });
                }
            }

            this.props.navigation.navigate("AspectInsertResult", {resultList: resultList});
        } else {
            let request = this.state.requestQueue.pop();
            this.waitForResponse(request, 0);
        }
    }

    /**
     * Check if a response is published through MQTT broker
     * @param request: list of request objects
     * @param seconds: how many seconds to wait for a response
     */
    waitForResponse(request, seconds) {
        let aspectTypes = MeasurementUnitService.getSensorMap(this.props.messages);

        // wait up to 10 seconds for a response
        if (seconds >= 5) {
            // if no response arrived, then return unsuccessful message
            let responseNotReceived = {
                aspectInsertResult: false,
                type: request.featureOfInterest,
                label: aspectTypes.get(request.featureOfInterest) ?? request.featureOfInterest,
                error: "Couldn't receive any response from the broker!"
            };
            this.setState({responseList: [...this.state.responseList, ...[responseNotReceived]]});
            this.scheduleNextRequest();
            return;
        }

        // Loop through responses and look for a match for the new request
        for (let i = 0; i < this.props.responseList.length; i++) {
            const response = this.props.responseList[i];
            let exists = false;
            if (response) {
                // groupingNotPossible case
                if (response.name && response.name === "groupingNotPossible") {
                    if (response.requestId && response.requestId === request.uuid) {
                        let response = {
                            aspectInsertResult: false,
                            type: request.featureOfInterest,
                            label: aspectTypes.get(request.featureOfInterest),
                            error: "Grouping is not possible"
                        };
                        this.setState({
                            monitoredAspectList: this.getMonitoredAspectList(),
                            responseList: [...this.state.responseList, ...[response]]
                        })
                        this.scheduleNextRequest();
                        return;
                    }
                } else {
                    // successful case
                    Object.keys(response.grouping).forEach(key => {
                        if (key === request.uuid) {
                            exists = true;
                        }
                    });
                }
            }

            if (exists && response) {
                // extract list of sensors used to process describe request
                let sensorList = [];

                Object.keys(response.grouping).forEach(requestId => {
                    if (requestId === request.uuid) {
                        Object.keys(response.grouping[requestId]).forEach(aggregationId => {
                            response.grouping[requestId][aggregationId]["sensor_list"].forEach(sensorObjectId => {
                                this.props.deviceList.forEach(device => {
                                    if (device.objectId === sensorObjectId) {
                                        sensorList.push(device.name);
                                    }
                                });
                            })
                        });
                    }
                });

                this.setState({isMounted: true, monitoredAspectList: this.getMonitoredAspectList()});

                // navigate to describe result page with success message
                let coatyResponse = {
                    aspectInsertResult: true,
                    type: request.featureOfInterest,
                    function: this.props.messages.meanFunction,
                    sensors: sensorList
                };
                this.setState({responseList: [...this.state.responseList, ...[coatyResponse]]})
                this.scheduleNextRequest();
                return;
            }
        }

        setTimeout(() => {
            this.waitForResponse(request, ++seconds);
        }, 1000);
    }

    /**
     * Delete a describe request from the list
     * @param requestId
     */
    deleteAspect = (requestId) => {
        // create a delete request for mqtt broker
        let deleteRequest = {
            "requestId": requestId
        }

        // send delete request
        if (MQTTClientService.mqtt.isConnected()) {
            MQTTClientService.send(Constants.constants.DELETE_REQUEST_TOPIC, JSON.stringify(deleteRequest));
        } else {
            ToastAndroid.showWithGravityAndOffset(this.props.messages.notConnectedToBroker, ToastAndroid.LONG, ToastAndroid.TOP, 0, 0);
            return;
        }

        // delete related observations
        this.props.observationList.forEach((observation) => {
            if (observation.responseId === requestId) {
                store.dispatch({type: "REMOVE_OBSERVATION", observation: observation});
            }
        });

        store.dispatch({type: "REMOVE_REQUEST_FROM_RESPONSE", requestId: requestId});
        store.dispatch({
            type: "REMOVE_REQUEST",
            request: AspectService.findRequestById(requestId, this.props.requestList)
        });

        setTimeout(() => {
            this.setState({monitoredAspectList: this.getMonitoredAspectList()});
        }, 750);

        ToastAndroid.showWithGravityAndOffset(this.props.messages.deleteAspectRequest, ToastAndroid.LONG, ToastAndroid.TOP, 0, 0);
    }

    /**
     * get list of monitored aspects
     */
    getMonitoredAspectList() {
        let monitoredAspects = [];
        for (let response of this.props.responseList) {
            let exists = false;
            monitoredAspects.forEach(aspect => {
                if (JSON.stringify(aspect) === JSON.stringify(response)) {
                    exists = true;
                }
            })
            if (!exists) {
                monitoredAspects.push(response);
            }
        }

        return monitoredAspects;
    }

    /**
     * Create a monitored aspect list item, e.g. $(temperature) aspect $(perRoom) with $(30) $(celcius) threshold.
     * @param requestId
     */
    getMonitoredAspectListText(requestId) {
        let monitoredAspectListItem = this.props.messages.monitoredAspectListItem;
        let exists = false;
        let isExternal = false;
        this.props.requestList.forEach((request) => {
            if (request.uuid === requestId) {
                monitoredAspectListItem = monitoredAspectListItem.replace("$", request.featureOfInterest);
                monitoredAspectListItem = monitoredAspectListItem.replace("$", request.location);
                monitoredAspectListItem = monitoredAspectListItem.replace("$", request.threshold);
                monitoredAspectListItem = monitoredAspectListItem.replace("$", request.unitOfMeasurement);
                exists = true;
                isExternal = request.isExternal;
            }
        });

        if (!exists) {
            return null;
        }

        return {
            item: monitoredAspectListItem,
            isExternal: isExternal
        };
    }

    /**
     * create the UI of domain selection card view
     * @returns {JSX.Element}
     */
    getDomainSelectionCardView() {
        // get the domain selection sentence
        let domainSelectionSentence = this.props.messages.domainSelectionSentence.split("$");

        // create picker entries for each of the domain type
        let domainSelectionSentenceItems = [];
        this.domainTypes.forEach((value, key) => {
            domainSelectionSentenceItems.push(<Picker.Item color={Colors.blue} label={value} key={key}
                                                           value={key}/>);
        });

        // create picker entries for each of the sub-domain
        let domainSelectionSentenceDetailsItems = [];
        MeasurementUnitService.getDomainMapByType(this.state.selectedDomain, this.props.messages).forEach((value, key) => {
            domainSelectionSentenceDetailsItems.push(<Picker.Item color={Colors.blue} label={value} key={key}
                                                                  value={key}/>);
        });

        // create picker for domain type selection
        let domainSelectionSentenceView = (
            <View style={styles.aspectPicker}>
                <Picker
                    mode="dropdown"
                    dropdownIconColor={Colors.blue}
                    selectedValue={this.state.selectedDomain}
                    onValueChange={(itemValue, itemPosition) => {
                        this.setState({selectedDomain: itemValue});
                    }}>
                    {domainSelectionSentenceItems}
                </Picker>
            </View>
        );

        // create picker for sub-domain type selection
        let domainSelectionSentenceDetailView = (
            <View style={styles.aspectPicker}>
                <Picker
                    mode="dropdown"
                    dropdownIconColor={Colors.blue}
                    selectedValue={this.state.selectedDomainDetail}
                    onValueChange={(itemValue, itemPosition) => {
                        this.setState({selectedDomainDetail: itemValue});
                    }}>
                    {domainSelectionSentenceDetailsItems}
                </Picker>
            </View>
        );

        // create final ui component and return
        return (
            <View
                style={styles.sentenceContainer}>
                <Text style={styles.sentence}>
                    {domainSelectionSentence[0]}
                </Text>
                {domainSelectionSentenceView}
                <Text style={styles.sentence}>
                    {" " + domainSelectionSentence[1]}
                </Text>
                {domainSelectionSentenceDetailView}
                <Text style={styles.sentence}>
                    {domainSelectionSentence[2]}
                </Text>
            </View>
        );
    }

    /**
     * create the UI of monitoring aspects' cardview, second cardview on UI
     * @returns {JSX.Element}
     */
    getMonitoringAspectCardView() {
        // get the sentence from language file
        let monitoringAspectSentence = this.props.messages.monitoringAspectSentence.split("$");

        // create picker entries for monitoring aspect type, places or objects
        let monitoringAspectSentenceItems = [];
        this.monitoringAspectSentenceTypes.forEach((value, key) => {
            monitoringAspectSentenceItems.push(<Picker.Item color={Colors.blue} label={value} key={key}
                                                            value={key}/>);
        });

        // create picker for each sub-type, e.g. space for places
        let monitoringAspectSentenceDetailsItems = [];
        MeasurementUnitService.getMonitoringAspectMapByType(this.state.selectedMonitoringAspect, this.props.messages).forEach((value, key) => {
            monitoringAspectSentenceDetailsItems.push(<Picker.Item color={Colors.blue} label={value} key={key}
                                                                   value={key}/>);
        });

        // create a picker for monitoring aspect
        let monitoringAspectSentenceView = (
            <View style={styles.aspectPicker}>
                <Picker
                    mode="dropdown"
                    dropdownIconColor={Colors.blue}
                    selectedValue={this.state.selectedMonitoringAspect}
                    onValueChange={(itemValue, itemPosition) => {
                        this.setState({selectedMonitoringAspect: itemValue});
                    }}>
                    {monitoringAspectSentenceItems}
                </Picker>
            </View>
        );

        // create a picker for sub-types of monitoring aspect
        let monitoringAspectSentenceDetailView = (
            <View style={styles.aspectPicker}>
                <Picker
                    mode="dropdown"
                    dropdownIconColor={Colors.blue}
                    selectedValue={this.state.selectedMonitoringAspectDetail}
                    onValueChange={(itemValue, itemPosition) => {
                        this.setState({selectedMonitoringAspectDetail: itemValue});
                    }}>
                    {monitoringAspectSentenceDetailsItems}
                </Picker>
            </View>
        );

        // create final ui component and return
        return (
            <View
                style={styles.sentenceContainer}>
                <Text style={styles.sentence}>
                    {monitoringAspectSentence[0]}
                </Text>
                {monitoringAspectSentenceView}
                <Text style={styles.sentence}>
                    {" " + monitoringAspectSentence[1]}
                </Text>
                {monitoringAspectSentenceDetailView}
                <Text style={styles.sentence}>
                    {monitoringAspectSentence[2]}
                </Text>
            </View>
        );
    }

    /**
     * Create UI component for notification creation sentence
     */
    getDescribeSentenceCardView() {
        const unitOfMeasurement = MeasurementUnitService.getUnitOfMeasurementByAspect(this.state.selectedMeasurementType);
        const areaOfInterest = this.getAreaOfInterestList();

        // get the sentence from language file
        let measurementTypeSentence = this.props.messages.measurementTypeSentence.split("$");

        // Create a picker from list of area of interests
        let areaOfInterestItems = areaOfInterest.map((s, i) => {
            return <Picker.Item color={Colors.blue} key={i} value={s} label={s}/>
        });

        // Create a picker from list of sensors
        let detailedAspectView = (
            <DropDownPicker
                items={this.state.sensorList}
                controller={instance => this.measurementTypePickerController = instance}
                defaultValue={this.state.selectedMeasurementType}
                containerStyle={styles.dropdownPickerContainer}
                style={styles.dropdownPicker}
                itemStyle={styles.dropdownPickerItem}
                placeholder={this.props.messages.search}
                placeholderStyle={styles.dropdownPickerText}
                searchable={true}
                dropDownStyle={styles.dropdownPickerDropdown}
                globalTextStyle={styles.dropdownPickerText}
                onChangeItem={item => {
                    this.setState({selectedMeasurementType: item.label})
                }}
                onSearch={text => {
                    let sensorList = this.state.sensorList;

                    sensorList.push({
                        value: text,
                        label: text
                    });

                    for (let i = 0; i < sensorList.length; i++) {
                        if (sensorList[i].label === this.state.previousText) {
                            sensorList.splice(i, 1);
                        }
                    }

                    this.setState({sensorList: sensorList, previousText: text});
                }}
            />
        );

        // create a view for threshold entry area
        let thresholdView = (
            <View style={styles.thresholdView}>
                <TextInput
                    style={styles.thresholdTextInput}
                    value={this.state.threshold}
                    onChange={(event) => this.setState({threshold: event.nativeEvent.text})}/>
            </View>
        );

        // create a picker for area of interest entry
        let areaOfInterestView = (
            <View style={styles.areaOfInterestPicker}>
                <Picker
                    mode="dropdown"
                    selectedValue={this.state.areaOfInterest}
                    dropdownIconColor={Colors.blue}
                    onValueChange={(itemValue) => {
                        this.setState({areaOfInterest: itemValue});
                    }}>
                    {areaOfInterestItems}
                </Picker>
            </View>
        );

        // create and return final UI component
        return (
            <View style={styles.sentenceContainer}>
                <Text style={styles.sentence}>
                    {measurementTypeSentence[0]}
                </Text>
                {detailedAspectView}
                <Text style={styles.sentence}>
                    {measurementTypeSentence[1]}
                </Text>
                {areaOfInterestView}
                <Text style={styles.sentence}>
                    {measurementTypeSentence[2]}
                </Text>
                {thresholdView}
                <Text style={styles.sentence}>
                    {" " + unitOfMeasurement}
                </Text>
                <Text style={styles.sentence}>
                    {measurementTypeSentence[3] + "."}
                </Text>
            </View>
        );
    }

    /**
     * Return the UI element of wsdl/owl file picker
     * @returns {JSX.Element}
     */
    filePickerModal() {
        return (
            <View style={styles.horizontalLayout}>
                <Text style={styles.serviceRequestLinkText}>{this.props.messages.fileName}: </Text>
                <DropDownPicker
                    items={this.state.fileList}
                    controller={instance => this.fileNamePickerController = instance}
                    defaultValue={this.state.fileName}
                    containerStyle={styles.fileNameDropdownPickerContainer}
                    style={styles.dropdownPicker}
                    itemStyle={styles.fileNameDropdownPickerItem}
                    placeholder={this.props.messages.search}
                    placeholderStyle={styles.dropdownPickerText}
                    searchable={true}
                    dropDownStyle={styles.dropdownPicker}
                    globalTextStyle={styles.dropdownPickerText}
                    onChangeItem={item => {
                        this.setState({fileName: item.label});
                    }}
                    onSearch={text => {
                        let fileList = this.state.fileList;
                        fileList.push({

                            value: text,
                            label: text
                        });
                        for (let i = 0; i < fileList.length; i++) {
                            if (fileList[i].label === this.state.previousFileName) {
                                fileList.splice(i, 1);
                            }
                        }

                        this.setState({fileList: fileList, previousFileName: text});
                    }}
                />
            </View>
        );
    }

    render() {
        let measurementTypeSentenceView = this.getDescribeSentenceCardView();

        return (
            <SafeAreaView style={styles.container}>
                <ScrollView style={styles.container} nestedScrollEnabled={true} keyboardShouldPersistTaps='always'>
                    <View style={styles.title}>
                        <Text style={styles.describe}>
                            {this.props.messages.describe}
                        </Text>
                        <Text style={styles.centeredText}>
                            {this.props.messages.describePageDefinition}
                        </Text>
                    </View>
                    <View style={styles.describeArea}>
                        {measurementTypeSentenceView}
                    </View>
                    <View style={styles.serviceRequestViaFileContainer}>
                        <CheckBox
                            tintColors={{true: Colors.blue, false: Colors.blue}}
                            value={this.state.serviceRequestViaFile}
                            onValueChange={() => this.setState({
                                serviceRequestViaFile: !this.state.serviceRequestViaFile,
                            })}/>
                        <TouchableOpacity
                            onPress={() => this.setState({
                                serviceRequestViaFile: !this.state.serviceRequestViaFile,
                            })}>
                            <Text style={styles.underlinedLeftAlignedText}>{this.props.messages.requestByUrl}</Text>
                        </TouchableOpacity>
                    </View>
                    {this.state.serviceRequestViaFile ? <View>
                        <View style={styles.requestByFileContainer}>
                            <Text
                                style={styles.requestByFileContainerDescription}>{this.props.messages.requestByFileDescription}</Text>
                            <View style={styles.horizontalLayout}>
                                <TouchableOpacity onPress={async () => {
                                    const pickerResult = await DocumentPicker.pickSingle({
                                        presentationStyle: 'fullScreen',
                                        copyTo: 'cachesDirectory',
                                    });
                                    this.setState({
                                        pickedFileName: pickerResult.name,
                                        pickedFileURI: pickerResult.fileCopyUri
                                    })
                                }}>
                                    <Text style={styles.selectFileButton}>{this.props.messages.select}</Text>
                                </TouchableOpacity>
                                <Text style={styles.pickedFileName}>{this.state.pickedFileName}</Text>
                            </View>
                        </View>
                    </View> : <View/>}
                    <View style={styles.submitArea}>
                        <TouchableOpacity

                            onPress={() => {
                                this.submit(this.state.selectedMeasurementType);
                            }}>
                            <View style={styles.submitButton}>
                                <Text style={styles.submitText}>{this.props.messages.submit}</Text>
                            </View>
                        </TouchableOpacity>
                    </View>
                    <View style={styles.monitoredAspectsView}>
                        <Text
                            style={styles.monitoredAspectDescription}>{this.props.messages.monitoredAspects}</Text>
                        <ScrollView
                            nestedScrollEnabled={true}
                            keyboardShouldPersistTaps='always'
                            style={styles.monitoredAspectList}>
                            {this.state.monitoredAspectList.map((response, index) => {
                                let items = [];
                                let incrementalKey = 1;

                                if (response && response.grouping) {
                                    Object.keys(response.grouping).forEach(requestId => {
                                        let monitoredAspect = this.getMonitoredAspectListText(requestId);

                                        if (monitoredAspect) {
                                            items.push(<View key={incrementalKey++}
                                                             style={styles.monitoredAspectListItem}>
                                                <Text style={styles.monitoredAspectText}>
                                                    {monitoredAspect.item}
                                                </Text>
                                                {monitoredAspect.isExternal ? (
                                                    <View style={styles.externalServiceIcon}>
                                                        <Tooltip
                                                            withPointer={true}
                                                            pointerColor={Colors.blue}
                                                            containerStyle={styles.tooltipContainer}
                                                            skipAndroidStatusBar={true}
                                                            withOverlay={false}
                                                            backgroundColor={Colors.light_blue}
                                                            popover={
                                                                <Text style={styles.toolTipText}>
                                                                    {this.props.messages.externalServiceRequest}
                                                                </Text>}>
                                                            <Icon color={Colors.blue} name={"info"} size={20}/>
                                                        </Tooltip>
                                                    </View>
                                                ) : <View style={styles.externalServiceIcon}/>}
                                                <TouchableOpacity style={styles.deleteIcon}
                                                                  onPress={() => this.deleteAspect(requestId)}>
                                                    <Icon name="trash" color={Colors.blue} size={30}/>
                                                </TouchableOpacity>
                                            </View>);
                                        }
                                    })
                                }

                                return items;
                            })}
                        </ScrollView>
                    </View>
                </ScrollView>
                <BottomNavigation navigation={this.props.navigation}/>
            </SafeAreaView>
        );
    }
}

// CSS style
let
    styles = EStyleSheet.create({
        container: {
            flex: 1,
            flexDirection: 'column',
            backgroundColor: "#FFF"
        },
        title: {
            width: "100%",
            flex: 1,
            justifyContent: "center",
            marginBottom: "10rem"
        },
        describe: {
            textAlign: 'center',
            fontSize: '30rem',
            fontWeight: 'bold',
            color: Colors.blue
        },
        centeredText: {
            textAlign: 'center',
            fontSize: '16rem',
            marginHorizontal: '10%',
            color: Colors.blue,
        },
        describeArea: {
            marginHorizontal: "10rem",
            marginVertical: "10rem",
            flex: 2,
        },
        aspectPicker: {
            width: "170rem",
            height: "30rem",
            borderWidth: "1rem",
            justifyContent: "center",
            marginVertical: '5%',
            borderColor: Colors.blue,
            borderRadius: "5rem"
        },
        thresholdView: {
            width: "70rem",
            height: "35rem",
            borderWidth: "1rem",
            borderColor: Colors.blue,
            borderRadius: "5rem"
        },
        thresholdTextInput: {
            textAlign: "center",
            textAlignVertical: "center",
            fontSize: "15rem",
            color: Colors.blue,
            padding: "5rem"
        },
        updateSchedulePicker: {
            width: "150rem",
            height: "30rem",
            borderWidth: "1rem",
            justifyContent: "center",
            marginVertical: '5rem',
            borderColor: Colors.blue
        },
        areaOfInterestPicker: {
            width: "150rem",
            height: "30rem",
            fontSize: "14rem",
            borderWidth: "1rem",
            justifyContent: "center",
            marginVertical: '5rem',

            borderColor: Colors.blue,
            borderRadius: "5rem"
        },
        dropdownPickerContainer: {
            marginVertical: "3rem",
            width: "150rem",
            height: "35rem",
            marginHorizontal: "5rem"
        },
        fileNameDropdownPickerContainer: {
            width: "200rem",
            height: "40rem",
        },
        dropdownPicker: {
            backgroundColor: Colors.light_blue,
            borderWidth: "1rem",
            borderColor: Colors.blue,
        },
        dropdownPickerItem: {
            backgroundColor: Colors.light_blue,
            justifyContent: "flex-start",
            width: "150rem",
            height: "35rem"
        },
        fileNameDropdownPickerItem: {
            backgroundColor: Colors.light_blue,
            justifyContent: "flex-start",
            marginBottom: "10rem",
            width: "150rem",
            height: "45rem"
        },
        dropdownPickerDropdown: {
            backgroundColor: Colors.light_blue,
            borderWidth: "1rem",
            borderColor: Colors.blue,
            width: "150rem",
            height: "120rem",
        },
        dropdownPickerText: {
            fontSize: "12rem",
            color: Colors.blue,

        },
        serviceRequestViaFileContainer: {
            marginHorizontal: "10rem",
            width: "100%",
            flexDirection: "row",
            alignItems: "flex-start",
        },
        submitArea: {
            flex: 1,
            alignItems: "center"
        },
        submitButton: {
            marginVertical: "3%",
            borderWidth: "1rem",
            borderRadius: "5rem",
            width: "100rem",
            height: "30rem",
            justifyContent: "center",
            alignItems: "center",
            alignSelf: "center",
            borderColor: Colors.blue
        },
        submitText: {
            color: Colors.blue,
            fontSize: "12rem"
        },
        sentence: {
            fontSize: "15rem",
            color: Colors.blue
        },
        sentenceContainer: {
            borderRadius: "5rem",
            overflow: "visible",
            padding: "5rem",
            flexDirection: "row",
            flexWrap: "wrap",
            margin: "10rem",
            alignItems: "center",
            justifyContent: "center",
            backgroundColor: Colors.light_blue
        },
        monitoredAspectsView: {
            flex: 4,
            marginHorizontal: "10rem",
            fontSize: "12rem",
            height: "300rem",
            width: "100%",
            alignSelf: "center"
        },
        monitoredAspectDescription: {
            marginTop: "30rem",
            marginVertical: "15rem",
            textAlign: 'center',
            fontSize: "15rem",
            color: Colors.blue
        },
        monitoredAspectList: {
            borderWidth: "1rem",
            marginBottom: "5rem",
            marginHorizontal: "10rem",
            height: "200rem",
            borderRadius: "5rem",
            borderColor: Colors.blue
        },
        monitoredAspectListItem: {
            justifyContent: "center",
            margin: "5rem",
            flexDirection: "row",
            borderRadius: "5rem",
            backgroundColor: Colors.light_blue
        },
        monitoredAspectText: {
            margin: "5rem",
            fontSize: "15rem",
            width: "50%",
            color: Colors.blue
        },
        serviceRequestTextInput: {
            paddingLeft: "10rem",
            color: Colors.blue,
            width: "200rem",
            height: "40rem",
            borderWidth: "1rem",
            borderColor: Colors.blue,
            borderRadius: "5rem",
        },
        checkbox: {
            color: Colors.blue
        },
        deleteIcon: {
            width: "25%",

            alignItems: "center",
            justifyContent: "center"
        },
        underlinedLeftAlignedText: {
            textDecorationLine: "underline",
            textAlign: "left",
            marginLeft: "10rem",
            fontSize: "15rem",
            color: Colors.blue
        },
        requestByURLFileNameContainer: {
            flexDirection: "column",
            backgroundColor: Colors.light_blue,
            height: "160rem",
            marginHorizontal: "10rem",
            borderRadius: "5rem",
            alignItems: "flex-start",
        },
        requestByFileContainer: {
            flexDirection: "column",
            backgroundColor: Colors.light_blue,
            height: "100rem",
            marginHorizontal: "20rem",
            padding: "5rem",
            borderRadius: "5rem",
            alignItems: "flex-start",
        },
        horizontalLayout: {
            marginVertical: "5rem",
            flexDirection: "row",
            justifyContent: "center",
            alignItems: "flex-start",
        },
        serviceRequestLinkText: {
            width: "150rem",
            textAlignVertical: "center",
            color: Colors.blue,
            height: "40rem",
            fontSize: "14rem",
            marginLeft: "5rem"
        },
        serviceRequestViaFileCheckbox: {
            borderColor: Colors.blue,
            borderWidth: "1rem"
        },
        externalServiceIcon: {
            width: "30rem",
            justifyContent: "center"
        },
        toolTipText: {
            fontSize: "12rem",
            color: Colors.blue
        },
        tooltipContainer: {
            borderWidth: "1rem",
            borderColor: Colors.blue,
            width: "150rem",
            height: "40rem"
        },
        requestByFileContainerDescription: {
            color: Colors.blue,
            textAlign: "center",
            fontSize: "15rem",
            margin: "5rem"
        },
        selectFileButton: {
            backgroundColor: Colors.blue,
            color: "white",
            borderRadius: "5rem",
            padding: "10rem",
            marginLeft: "80rem",
            marginTop: "10rem"
        },
        pickedFileName: {
            width: "150rem",
            marginLeft: "20rem",
            color: Colors.blue,
            fontSize: "15rem",
            alignSelf: "center",
        }
    });

// connect to react redux
const
    mapStateToProps = (state) => ({
        messages: state.language.messages,
        requestList: state.request.requestList,
        responseList: state.response.responseList,
        deviceList: state.device.deviceList,
        observationList: state.observation.observationList
    });

const
    mapDispatchToProps = {};

export default connect(mapStateToProps, mapDispatchToProps)

(
    Describe
)
;
