/**
 * Copyright (C) 2021 fortiss GmbH
 * @author Erkan Karabulut – "karabulut@fortiss.org"
 * @version 1.2
 * This file includes device discovery page's UI and business logic
 */

import React from 'react';
import store from "../config/store";
import {View, Text, SafeAreaView, TextInput, FlatList, TouchableOpacity, Modal, Image, ScrollView} from 'react-native';
import {connect} from 'react-redux';
import EStyleSheet from 'react-native-extended-stylesheet';
import BottomNavigation from "../component/BottomNavigation";
import DeviceService from "../service/DeviceService";
import {bindActionCreators} from "redux";
import {addDevice, removeDevice} from "../action/device";
import * as Colors from "../constant/string/Colors";
import {Tooltip} from "react-native-elements";
import MQTTClientService from "../service/MQTTClientService";

/**
 * This class is used to discover alive sensors and list them in the UI
 */
class Discover extends React.Component {
    constructor(props) {
        super(props);
        this.state = {
            deviceList: [],
            searchValue: "",
            mounted: false,
            deviceDetailModalVisible: false,
            thingName: null,
            location: null,
            unit: null,
            brokerState: false
        };
    }

    /**
     * Check if the mobile app connected to a broker or not
     */
    checkBrokerState() {
        this.setState({brokerState: MQTTClientService.mqtt && MQTTClientService.mqtt.isConnected()});
        this.brokerStateTimer = setTimeout(() => {
            this.checkBrokerState();
        }, 1000);
    }

    /**
     * This method is triggered when the search text is changed
     * @param newSearchValue
     */
    updateSearchText = (newSearchValue) => {
        // set new search value in the state
        let text = newSearchValue.nativeEvent.text;
        this.setState({searchValue: text});
        if (text === null || text === undefined || text === "") {
            // if the search value is empty, then show all devices
            this.setState({deviceList: this.props.deviceList});
        } else {
            // if not, then filter the devices according to their names and descriptions
            let newDeviceList = [];
            for (let i = 0; i < this.props.deviceList.length; i++) {
                if (this.props.deviceList[i].name.toLowerCase().includes(text.toLowerCase()) ||
                    this.props.deviceList[i].description.toLowerCase().includes(text.toLowerCase())) {
                    newDeviceList.push(this.props.deviceList[i]);
                }
            }
            this.setState({deviceList: newDeviceList});
        }
    };

    componentDidMount() {
        this.setState({deviceList: this.props.deviceList, mounted: true});
        // subscribe to redux store changes to detect newly discovered devices
        this.unsubscribe = store.subscribe(() => {
            if (this.state.mounted === true) {
                this.setState({deviceList: store.getState().device.deviceList})
            }
        });

        // check every second if the app is connected to the broker
        this.brokerStateTimer = setTimeout(() => {
            this.checkBrokerState();
        }, 1000);
    }

    componentWillUnmount() {
        // unsubscribe from react redux storage subscription
        this.unsubscribe();
        clearTimeout(this.brokerStateTimer);
    }

    /**
     * This method is triggered when user clicked on a device
     * @param device
     */
    showDeviceDetailModal(device) {
        this.setState({
            deviceDetailModalVisible: true,
            thing: device.thing,
            location: DeviceService.getLocationTextFromDeviceDefinition(device),
            unit: device.unit,
        });
    }

    /**
     * Render a device item in device list
     * @param item: Device item details
     * @returns {JSX.Element}
     */
    renderItem({item}) {
        return (
            <TouchableOpacity onPress={() => this.showDeviceDetailModal(item)}>
                <View style={styles.device}>
                    <View style={styles.deviceListColumnLeft}>
                        <Text style={styles.deviceName}>{item.name}</Text>
                        {DeviceService.getDeviceIconByType(item.name)}
                    </View>
                    <View style={styles.deviceListColumnRight}>
                        <Text style={styles.deviceDescriptionText}>Description</Text>
                        <Text style={styles.deviceDescription}>{item.description}</Text>
                    </View>
                </View>
            </TouchableOpacity>
        );
    }

    /**
     * Renders a modal which includes detailed information about the selected device/sensor
     */
    renderDeviceDetailsModal() {
        return (
            <Modal
                animationType="slide"
                transparent={true}

                visible={this.state.deviceDetailModalVisible}>
                <View style={styles.modalView}>
                    <Text style={styles.modalHeader}>{this.props.messages.deviceDetails}</Text>
                    <View style={styles.modalRowView}>
                        <Text style={styles.boldText}>{this.props.messages.thing}: </Text>
                        <Text style={styles.text}>{this.state.thing}</Text>
                    </View>
                    <View style={styles.modalRowView}>
                        <Text style={styles.boldText}>{this.props.messages.location}: </Text>
                        <Text style={styles.text}>{this.state.location}</Text>
                    </View>
                    <View style={styles.modalRowView}>
                        <Text style={styles.boldText}>{this.props.messages.unit}: </Text>
                        <Text style={styles.text}>{this.state.unit}</Text>
                    </View>
                    <TouchableOpacity style={styles.leftAlignment}
                                      onPress={() => this.setState({deviceDetailModalVisible: false})}>
                        <Text style={styles.modalCloseButton}>{this.props.messages.close}</Text>
                    </TouchableOpacity>
                </View>
            </Modal>
        );
    }

    render() {
        return (
            <SafeAreaView style={styles.container}>
                {this.renderDeviceDetailsModal()}
                <View style={styles.title}>
                    <Text style={styles.discover}>
                        {this.props.messages.discover}
                    </Text>
                    <Text style={styles.discoverPageHeader}>
                        {this.props.messages.discoverPageHeader}
                    </Text>
                </View>
                <View style={styles.searchBarView}>
                    <TextInput
                        value={this.state.searchValue}
                        placeholderTextColor={Colors.blue}
                        placeholder={this.props.messages.searchBarPlaceHolder}
                        onChange={this.updateSearchText} style={styles.searchBar}/>
                    <Text style={styles.totalNumberOfDevices}>
                        {this.props.messages.connectionStatus}: {(this.state.brokerState ? this.props.messages.connected : this.props.messages.disconnected) + "\n"}
                        {this.props.messages.deviceCount}: {this.state.deviceList.length}
                    </Text>
                </View>

                <View style={styles.deviceList}>
                    <FlatList
                        data={this.state.deviceList}
                        renderItem={(item) => this.renderItem(item)}
                        keyExtractor={item => item.objectId}
                    />
                </View>
                <BottomNavigation navigation={this.props.navigation}/>
            </SafeAreaView>
        );
    }
}

// CSS style
let styles = EStyleSheet.create({
    container: {
        flex: 1,
        flexDirection: 'column',
        backgroundColor: "#FFF"
    },
    title: {
        width: "100%",
        flex: 1,
        justifyContent: "center",
        marginBottom: "10rem",
    },
    discover: {
        textAlign: 'center',
        fontSize: '30rem',
        fontWeight: 'bold',
        color: Colors.blue
    },
    discoverPageHeader: {
        textAlign: 'center',
        fontSize: '16rem',
        marginHorizontal: '10%',
        color: Colors.blue,

    },
    searchBarView: {
        flexDirection: "row",
        marginHorizontal: "5%",
        marginVertical: "10rem",
        width: "100%",
    },
    searchBar: {
        alignItems: "flex-start",
        width: "170rem",
        height: "40rem",
        borderWidth: "1rem",
        borderRadius: "5rem",
        fontSize: "12rem",
        textAlign: "left",
        padding: "5rem",
        borderColor: Colors.blue,
        color: Colors.blue
    },
    totalNumberOfDevices: {
        justifyContent: "flex-end",
        color: Colors.blue,
        fontSize: "12rem",
        width: "170rem",
        textAlign: "right"

    },
    deviceList: {
        flex: 5,
        marginBottom: "20rem",
        borderWidth: "1rem",
        borderColor: Colors.blue,
        marginHorizontal: "20rem",
        borderRadius: "5rem"
    },
    device: {
        flexDirection: 'row',
        marginHorizontal: "10rem",
        marginVertical: "10rem",
        padding: "5rem",
        borderRadius: "5rem",
        backgroundColor: Colors.light_blue
    },
    deviceName: {
        textAlign: 'center',
        fontSize: "14rem",
        fontWeight: 'bold',
        color: Colors.blue
    },
    deviceDescriptionText: {
        fontWeight: 'bold',
        color: Colors.blue
    },
    deviceListColumnRight: {
        marginHorizontal: "5rem",
        width: "200rem"
    },
    deviceListColumnLeft: {
        marginHorizontal: "5rem",
        alignItems: "center",
        justifyContent: "center",
        width: "30%"
    },
    deviceDescription: {
        width: "60%",
        fontSize: "12rem",
        color: Colors.blue
    },
    modalView: {
        marginVertical: "3%",
        width: "300rem",
        height: "180rem",
        alignSelf: "center",
        borderRadius: "10rem",
        borderColor: Colors.blue,
        borderWidth: "1rem",
        backgroundColor: "#FFF"
    },
    modalHeader: {
        fontSize: "20rem",
        fontWeight: "bold",
        textAlign: "center",
        margin: "10rem",
        color: Colors.blue
    },
    modalRowView: {
        flexDirection: "row",
        marginLeft: "10rem"
    },
    boldText: {
        fontSize: "12rem",
        fontWeight: "bold",
        color: Colors.blue
    },
    text: {
        fontSize: "12rem",
        color: Colors.blue,
        width: "220rem"
    },
    modalCloseButton: {
        padding: "10rem",
        backgroundColor: Colors.light_blue,
        borderRadius: "15rem",
        textAlign: "left",
        color: Colors.blue
    },
    leftAlignment: {
        alignItems: "flex-end",
        margin: "10rem"
    },

    brokerIcon: {
        alignItems: "flex-start",
        marginLeft: "10rem",
        marginTop: "10rem"
    }
});

// connect to react redux
const mapStateToProps = (state) => ({
    messages: state.language.messages,
    deviceList: state.device.deviceList
});

// get addDevice and removeDevice reducers
const mapDispatchToProps = dispatch => (
    bindActionCreators({
        addDevice, removeDevice
    }, dispatch)
);

export default connect(mapStateToProps, mapDispatchToProps)(Discover);
