/**
 * Copyright (C) 2021 fortiss GmbH
 * @author Erkan Karabulut – "karabulut@fortiss.org"
 * @version 1.2
 * Main/Home page of the app UI
 */

import React from 'react';
import {View, Text, Image, TouchableOpacity, Modal, ScrollView, TextInput, ToastAndroid} from 'react-native';
import {connect} from 'react-redux';
import EStyleSheet from 'react-native-extended-stylesheet';
import Constants from "../constant/constants";
import * as Colors from "../constant/string/Colors";
import Entypo from "react-native-vector-icons/Entypo";
import AntDesign from "react-native-vector-icons/AntDesign";
import Feather from "react-native-vector-icons/Feather";
import MQTTClientService from "../service/MQTTClientService";
import HtmlView from "react-native-htmlview";


/**
 * This class includes UI and business logic of the Home screen
 */
class Home extends React.Component {
    constructor(props) {
        super(props);
        this.state = {
            settingsModalVisible: false,
            showAdvancedSettings: false,
            advancedSettingsText: this.props.messages.showAdvancedSettings,
            settings: {},
            howToModalVisible: false
        };
    }

    /**
     * This method is triggered when user click on save settings button
     */
    setSettings() {
        this.setState({settingsModalVisible: false});
        // update constants
        Constants.updateConstants(this.state.settings);
        // reconnect to mqtt broker
        MQTTClientService.close();
        MQTTClientService.connect();
        ToastAndroid.showWithGravityAndOffset(this.props.messages.settingsSavedSuccessfully, ToastAndroid.LONG, ToastAndroid.TOP, 0, 0);
    }

    /**
     * close settings modal
     */
    cancelSettings() {
        this.setState({settingsModalVisible: false});
    }

    /**
     * render settings modal
     */

    settingsModal() {
        let settingsElements = this.props.constantsObject;
        return (
            <Modal
                animationType="slide"
                transparent={true}
                visible={this.state.settingsModalVisible}>
                <View style={styles.centeredView}>
                    <ScrollView style={styles.settingsModal}>
                        <View style={styles.settingsHeaderView}>
                            <Text style={styles.settingsHeaderText}>
                                {this.props.messages.settings}
                            </Text>
                        </View>
                        {
                            /* Create a row of key-value pairs for each of the constants */
                            Object.keys(settingsElements).map((key) => (
                                // First, show only broker settings unless user clicked on advanced settings
                                !key.includes("version") && (key.includes("BROKER") || this.state.showAdvancedSettings) ?
                                    <View key={key} style={styles.inputItem}>
                                        <Text style={styles.settingsKeyText}>
                                            {key.split("_").join(" ") + ": "}
                                        </Text>
                                        <TextInput style={styles.settingsInput}
                                                   secureTextEntry={key.includes("PASSWORD")}
                                                   defaultValue={settingsElements[key].toString()}
                                                   onChangeText={(text) => {
                                                       let newSettings = this.state.settings;
                                                       newSettings[key] = text;
                                                       this.setState({settings: newSettings});
                                                   }}/>
                                    </View> : <View key={key}/>
                            ))
                        }
                        {/* Advanced settings button */}
                        <View style={styles.settingsButtonView}>
                            <TouchableOpacity style={styles.settingsButton} onPress={() => {
                                this.cancelSettings()
                            }}>
                                <Text style={styles.cancelButton}>
                                    {this.props.messages.cancel}
                                </Text>
                            </TouchableOpacity>
                            <TouchableOpacity style={styles.settingsButton} onPress={() => {
                                this.setSettings()
                            }}>
                                <Text style={styles.saveButton}>
                                    {this.props.messages.save}
                                </Text>
                            </TouchableOpacity>
                        </View>
                    </ScrollView>
                </View>
            </Modal>
        );
    }

    howtoModal() {
        return (
            <Modal
                animationType="slide"
                transparent={true}
                visible={this.state.howToModalVisible}>
                <ScrollView style={styles.howToModal}>
                    <HtmlView stylesheet={EStyleSheet.create({
                        h4: {
                            textAlign: "center",
                            marginTop: "10rem",
                            fontWeight: "bold",
                            fontSize: "20rem",
                            color: Colors.blue
                        },
                        p: {
                            color: Colors.blue,
                            fontSize: "8rem",
                            marginHorizontal: "10rem"
                        }
                    })}
                              value={this.props.messages.howToModalText}/>
                    <TouchableOpacity onPress={() => this.setState({howToModalVisible: false})}>
                        <Text style={styles.closeHowToModalButton}>{this.props.messages.close}</Text>
                    </TouchableOpacity>
                </ScrollView>
            </Modal>
        )
    }

    render() {
        return (
            <View style={styles.container}>
                {this.howtoModal()}
                <View style={styles.topView}>
                    <TouchableOpacity style={styles.howtoIcon} onPress={() => {
                        this.setState({howToModalVisible: true})
                    }}>
                        <Entypo name={"help"} size={30} color={Colors.blue}/>
                    </TouchableOpacity>
                    <View style={styles.fortissLogoView}>
                        <Image source={require('../asset/image/fortiss.png')}
                               style={styles.fortissLogo}/>
                    </View>
                </View>
                <View style={styles.appTitle}>
                    <Text style={styles.appTitleShortText}>
                        {this.props.messages.appTitleShort}
                    </Text>
                    <Text style={styles.fullAppTitle} numberOfLines={1} adjustsFontSizeToFit={true}>
                        {this.props.messages.fullAppTitle}
                    </Text>
                </View>
                <View style={styles.navigations}>
                    <View style={styles.navigationItem}>
                        <TouchableOpacity style={styles.navigationIcon} onPress={() => {
                            MQTTClientService.close();
                            Constants.switchToIIoTLab();
                            MQTTClientService.connect();
                            this.props.navigation.navigate('Discover')
                        }}>
                            <Image source={require("../asset/image/Lab-Flask.png")} style={styles.navigationIcon}/>
                        </TouchableOpacity>
                        <Text style={styles.navigationItemTitle}>
                            {this.props.messages.iiotLab}
                        </Text>
                    </View>
                    <View style={styles.navigationItem}>
                        <TouchableOpacity onPress={() => {
                            MQTTClientService.close();
                            Constants.switchToEFPF();
                            MQTTClientService.connect();
                            this.props.navigation.navigate('Discover')
                        }}>
                            <Image source={require("../asset/image/efpf.png")} style={styles.navigationIcon}/>
                        </TouchableOpacity>
                        <Text style={styles.navigationItemTitle}>
                            {this.props.messages.efpfPlatform}
                        </Text>
                    </View>
                    <View style={styles.navigationItem}>
                        <TouchableOpacity onPress={() => {
                            MQTTClientService.close();
                            MQTTClientService.connect();
                            this.props.navigation.navigate('Discover')
                        }}>
                            <AntDesign name="filetext1" style={styles.navigationIcon} color={Colors.blue} size={50}/>
                        </TouchableOpacity>
                        <Text style={styles.navigationItemTitle}>
                            {this.props.messages.useFromSettings}
                        </Text>
                    </View>
                    {this.settingsModal()}
                </View>
                <TouchableOpacity style={styles.settingsIcon} onPress={() => {
                    this.setState({settingsModalVisible: true});
                }}>
                    <Feather name="settings" color={Colors.blue} size={40}/>
                </TouchableOpacity>
                <View style={styles.foot}>
                    <Text style={styles.versionText} adjustsFontSizeToFit={true}>
                        {"Version: " + Constants.constants.version}
                    </Text>
                </View>
            </View>
        );
    }
}

// CSS styles
let styles = EStyleSheet.create({
    container: {
        flex: 1,

        flexDirection: 'column',
        backgroundColor: "#FFF",
        fontFamily: "museo-sans"
    },
    topView: {
        flexDirection: "row",
        flex: 1,
    },
    fortissLogoView: {
        alignItems: 'flex-end',

    },
    fortissLogo: {

        marginTop: "10rem",
        marginRight: "20rem",
        width: "100rem",
        height: "30%",
        resizeMode: 'contain'
    },
    appTitle: {
        flex: 2,
        alignItems: 'center',
        justifyContent: 'center',
    },
    appTitleShortText: {
        width: "100%",
        fontSize: "35rem",
        textAlign: 'center',
        fontWeight: 'bold',
        color: Colors.blue,
        fontFamily: "Verdana"
    },
    fullAppTitle: {
        fontSize: "20rem",
        textAlign: 'center',
        fontWeight: "bold",
        color: Colors.blue,
        fontFamily: "MuseoSans"
    },

    navigations: {
        marginTop: "10rem",
        flex: 4,
        flexDirection: 'row',
        flexWrap: "wrap",
        justifyContent: 'center',
        alignItems: "center"

    },
    navigationIcon: {
        maxWidth: "60rem",
        maxHeight: "60rem",
        resizeMode: "contain",
    },
    navigationItem: {
        flex: 1,
        marginVertical: "10%",
        alignItems: 'center',
        justifyContent: 'center',

    },
    navigationItemTitle: {
        width: "90rem",
        fontWeight: 'bold',
        fontSize: "16rem",
        textAlign: "center",
        color: Colors.blue
    },
    foot: {
        flex: 1,
        flexDirection: "column-reverse",
    },
    settingsIcon: {
        alignItems: "flex-end",
        marginRight: "30rem",
    },
    settingsModal: {
        marginTop: "20rem",
        backgroundColor: "white",
        borderRadius: "10rem",
        borderColor: Colors.blue,
        borderWidth: "1rem",
        width: "350rem",
        height: "350rem",
        flexDirection: "column",
    },
    settingsButtonView: {
        flexDirection: "row",
        alignItems: "center",
        justifyContent: "center",
        marginTop: "40rem"
    },
    cancelButton: {
        backgroundColor: Colors.light_blue,
        borderColor: Colors.blue,
        borderWidth: "1rem",
        width: "100rem",
        textAlign: "center",
        marginHorizontal: "20rem",
        padding: "5rem",
        color: Colors.blue,
        borderRadius: "10rem"
    },
    saveButton: {
        backgroundColor: Colors.light_blue,
        borderColor: Colors.blue,
        borderWidth: "1rem",
        width: "100rem",
        textAlign: "center",
        marginHorizontal: "20rem",
        padding: "5rem",
        color: Colors.blue,
        borderRadius: "10rem",
    },
    centeredView: {
        justifyContent: "center",
        alignItems: "center"
    },
    settingsKeyText: {
        width: "130rem",
        fontSize: "12rem",
        color: Colors.blue
    },
    settingsInput: {
        fontSize: "12rem",
        borderBottomWidth: "1rem",
        width: "200rem",
        color: Colors.blue
    },
    inputItem: {
        alignItems: "center",
        justifyContent: "center",
        flexDirection: "row"
    },
    advancedSettingsView: {
        justifyContent: "flex-end",
        alignItems: "flex-end",
        margin: "15rem"
    },
    advancedSettingsText: {
        color: Colors.blue,
        fontSize: "14rem"
    },
    settingsHeaderView: {
        alignItems: "center",
        marginVertical: "15rem"
    },
    settingsHeaderText: {
        fontSize: "20rem",
        textAlign: "center",
        color: Colors.blue
    },
    versionText: {

        justifyContent: "flex-end",
        marginLeft: "5rem",
        marginBottom: "2rem",
        color: Colors.blue
    },
    howtoIcon: {
        flex: 1,
        marginTop: "10rem",
        marginLeft: "20rem",
        justifyContent: "flex-start"

    },
    howToModal: {
        backgroundColor: Colors.light_blue,
        marginVertical: "10%",
        width: "300rem",
        height: "300rem",
        maxHeight: "300rem",
        alignSelf: "center",
        borderRadius: "5rem"
    },
    howToModalText: {
        color: Colors.blue,
        fontSize: "30rem",
        textAlign: "left",
        margin: "10rem"
    },
    closeHowToModalButton: {
        color: "white",
        padding: "15rem",
        width: "100rem",
        borderRadius: "5rem",
        alignSelf: "center",
        textAlign: "center",
        marginVertical: "10%",
        backgroundColor: Colors.blue
    }
});

// connect to react redux
const mapStateToProps = (state) => ({
    messages: state.language.messages,
    deviceList: state.device.deviceList,
    constantsObject: state.constants.constantsObject
});


const mapDispatchToProps = {}

export default connect(mapStateToProps, mapDispatchToProps)(Home);
