/**
 * Copyright (C) 2021 fortiss GmbH
 * @author Erkan Karabulut – "karabulut@fortiss.org"
 * @version 1.2
 * This file contains UI and business logic of "Updates" page, which includes graphs of observations
 */

import React, {useState} from 'react';
import {View, Text, ScrollView, SafeAreaView, Button, TouchableOpacity} from 'react-native';
import {connect} from 'react-redux';
import BottomNavigation from "../component/BottomNavigation";
import EStyleSheet from 'react-native-extended-stylesheet';
import {LineChart, Grid, YAxis} from 'react-native-svg-charts'
import store from "../config/store";
import _, {zip} from "underscore";
import AspectService from "../service/AspectService";
import ObservationService from "../service/ObservationService";
import * as Colors from '../constant/string/Colors';

/**
 * Creates graphs for observation results
 */
class Updates extends React.Component {
    constructor(props) {
        super(props);
        this.state = {
            observationList: [],
            selectedAspectList: [],
            isMounted: false,
            graphVisible: false,
            statistics: [
                {
                    graphId: "",
                    min: 10,
                }
            ]
        };
        visibility = {
            visible: true
        };
    }

    componentDidMount() {
        // get observation list when the component is first mounted
        this.retrieveObservations();
        // subscribe to further observation changes
        this.unsubscribe = store.subscribe(() => {
            this.retrieveObservations();
        });
    }

    componentWillUnmount() {
        // unsubscribe from observation changes
        this.unsubscribe();
    }

    /**
     * Create a list of observations grouped by description/response and location
     */
    retrieveObservations() {
        // Group observations by requestId
        let groupedObservations = _.groupBy(this.props.observationList, function (observation) {
            return observation.requestId;
        });

        // group observations by location string
        _.forEach(groupedObservations, (value, key) => {
            groupedObservations[key] = _.groupBy(groupedObservations[key], function (observation) {
                return JSON.stringify(observation.location);
            });
        });

        // create lists of observations that are grouped by requestId, location
        let newObservationList = [];

        _.forEach(groupedObservations, (observationsByLocation, requestId) => {
            _.forEach(observationsByLocation, (observationList, location) => {
                // get most recent 10 observations
                let mostRecentObservations = _.sortBy(observationList, function (observation) {
                    return observation.timestamp;
                }).reverse().slice(0, 10);

                // sort them by timestamp
                let orderedObservations = _.sortBy(mostRecentObservations, function (observation) {
                    return observation.timestamp;
                });

                // extract observation values
                let observationValues = [];
                orderedObservations.map(observation => (
                    observationValues.push(observation.value)
                ));
                //const m = observationValues => Math.max(...observationValues);
                let max = Math.max(...observationValues);
                //const mi = observationValues => Math.min(...observationValues);
                let min = Math.min(...observationValues);
                let lastOne = observationValues[observationValues.length - 1];
                // create an object for the observation list that can be visualized
                let request = AspectService.findRequestById(observationList[0].requestId, this.props.requestList);
                if (request) {
                    let graphTitle = observationList[0]["location"];
                    newObservationList.push({
                        "title": graphTitle,
                        "values": observationValues,
                        "min": min,
                        "max": max,
                        "currently": lastOne
                    });
                }

            });
        });
        this.setState({observationList: newObservationList});

    }

    render() {
        const contentInset = {top: 20, bottom: 20}
        let graphIndex = 0;

        const textHidden = this.state.graphVisible ? "Hide Details" : "Show Details";


        // create a graph for each of the entry in observationList, which is grouped by location and requestId
        let renderList = this.state.observationList.map(observations => (

            <View key={graphIndex++}>
                <Text style={styles.graphTitle}>
                    {observations.title}
                </Text>

                <View key={observations.objectId}>
                    <View style={styles.statistic} numberOfLines={1}>
                        <Text style={styles.statisticText}>
                            Current: {observations.currently}
                        </Text>
                        <Text style={styles.statisticText}>
                            Maximum: {observations.max}

                        </Text>
                        <Text style={styles.statisticText}>
                            Minimum: {observations.min}

                        </Text>
                    </View>
                    <View style={styles.buttonContainer}>
                        <TouchableOpacity
                            onPress={() => {
                                this.setState({graphVisible: !this.state.graphVisible});
                            }}>
                            <Text style={styles.hideShowButton}>{textHidden}</Text>


                        </TouchableOpacity>
                    </View>
                    {this.state.graphVisible ?
                        <View key={observations.objectId} style={styles.graph}>
                            <YAxis
                                data={observations.values}
                                contentInset={contentInset}
                                svg={{
                                    fill: Colors.blue,
                                    fontSize: 12,
                                }}
                                numberOfTicks={10}
                                formatLabel={(value) => value}
                            />
                            <LineChart
                                style={styles.lineChart}
                                data={observations.values}
                                svg={{stroke: Colors.blue}}
                                contentInset={contentInset}
                            >
                                <Grid/>
                            </LineChart>
                        </View> : null}
                </View>
            </View>
        ));

        return (
            <SafeAreaView style={styles.container}>

                <ScrollView style={styles.graphViewContainer}>
                    <View style={styles.title}>
                        <Text style={styles.updates}>
                            {this.props.messages.getUpdates}
                        </Text>
                        <Text style={styles.pageDescription}>
                            {this.props.messages.getUpdatesDefinition}
                        </Text>
                    </View>

                    <View>
                        {renderList}
                    </View>
                </ScrollView>
                <View style={{flex: 1}}/>
                <BottomNavigation navigation={this.props.navigation}/>
            </SafeAreaView>
        );
    }
}

// CSS style
let styles = EStyleSheet.create({
    container: {
        flex: 1,
        backgroundColor: "#FFF",
        flexDirection: 'column',

    },
    title: {
        width: "100%",
        flex: 1,
        justifyContent: "center",
        marginBottom: "10rem"
    },
    updates: {
        textAlign: 'center',
        margin: '10rem',
        fontSize: '30rem',
        fontWeight: 'bold',
        marginHorizontal: '10%',

        color: Colors.blue
    },
    pageDescription: {
        textAlign: 'center',
        fontSize: '16rem',
        marginHorizontal: '15rem',
        color: Colors.blue
    },
    graphTitle: {
        textAlign: 'center',
        marginTop: '20rem',
        fontSize: '15rem',
        fontWeight: 'bold',
        color: Colors.blue,
        marginHorizontal: "10rem"
    },
    graph: {

        flexDirection: "row",
        marginHorizontal: "10%",
        height: "200rem",
        width: "80%",
        padding: "10rem",
        marginVertical: "1%",
        backgroundColor: Colors.light_blue,

    },
    statistic: {
        flexWrap: "wrap",
        marginHorizontal: "10%",
        flexDirection: "row",
        justifyContent: "center",
        width: "80%",
        padding: "10rem",
        marginTop: "3%",
        backgroundColor: Colors.light_blue


    },
    statisticText: {
        fontSize: "15rem",
        color: Colors.blue,
        marginVertical: "2%",
        marginHorizontal: "3%",
    },
    buttonContainer: {
        marginHorizontal: "10%",
        width: "80%",
        backgroundColor: Colors.light_blue,
        flexDirection: "row",
        justifyContent: "flex-end"


    },
    hideShowButton: {

        margin: "5%",
        fontSize: "10rem",
        color: Colors.blue
    },
    lineChart: {
        flex: 1,
        marginLeft: "15rem"
    },
    graphViewContainer: {
        marginVertical: "20rem"
    }
});

// connect to react redux
const mapStateToProps = (state) => ({
    messages: state.language.messages,
    observationList: state.observation.observationList,
    deviceList: state.device.deviceList,
    responseList: state.response.responseList,
    requestList: state.request.requestList
});

const mapDispatchToProps = {};

export default connect(mapStateToProps, mapDispatchToProps)(Updates);
