/**
 * Copyright (C) 2021 fortiss GmbH
 * @author Erkan Karabulut – "karabulut@fortiss.org"
 * @version 1.2
 * This file includes MQTT broker interactions
 */

import init from 'react_native_mqtt';
import uuid from 'react-native-uuid';
import AsyncStorage from "@react-native-async-storage/async-storage";
import Constants from "../constant/constants";
import store from '../config/store'
import PushNotification from "react-native-push-notification";
import DeviceService from "./DeviceService";
import AspectService from "./AspectService";

init({
    size: 10000,
    storageBackend: AsyncStorage,
    defaultExpires: 1000 * 3600 * 24,
    enableCache: true,
    sync: {},
});

const defaultConnectOptions = {
    reconnect: false,
    cleanSession: true,
    mqttVersion: 3,
    keepAliveInterval: 60,
    timeout: 60
}

/**
 * This class is used to connect/disconnect to/from an MQTT broker, send/receive messages and subscribe to topics
 */
export default class MQTTClientService {
    static mqtt = null;
    static QOS = 2;
    static RETAIN = false;

    /**
     * Connect to the mqtt broker
     */
    static connect() {
        // Get constants from react-redux storage and update Constants class
        Constants.updateConstants(store.getState().constants.constantsObject);
        let host = Constants.constants.BROKER_URL;
        let port = parseInt(Constants.constants.BROKER_PORT);
        let options = {
            userName: Constants.constants.BROKER_USERNAME,
            password: Constants.constants.BROKER_PASSWORD
        };

        // create a client ID based on current datetime
        let currentTime = +new Date();
        let clientID = currentTime + uuid.v1();
        clientID = clientID.slice(0, 23);

        this.mqtt = new Paho.MQTT.Client(host, port, clientID);
        this.mqtt.onConnectionLost = (res) => {
            MQTTClientService.onMQTTLost(res);
        };
        this.mqtt.onFailure = (res) => {
            MQTTClientService.onMQTTFailure(res);
        }
        this.mqtt.onMessageArrived = (message) => {
            MQTTClientService.onMQTTMessageArrived(message);
        };

        const connectOptions = options ? options : defaultConnectOptions;

        this.mqtt.connect({
            onSuccess: MQTTClientService.onMQTTSuccess,
            onFailure: MQTTClientService.onMQTTFailure,
            ...connectOptions
        });
    }

    /**
     * Successfully connected to a broker
     */
    static onMQTTSuccess = () => {
        MQTTClientService.onMQTTConnect()
    }

    /**
     * Failed while connecting to a broker or disconnected with a failure
     */
    static onMQTTFailure = (res) => {
        MQTTClientService.onMQTTLost(res);
    }

    /**
     * Subscribe to a given channel
     * @param channel
     */
    static subscribeChannel(channel) {
        console.log('Subscribe to channel: ', channel)
        if (!this.mqtt || !this.mqtt.isConnected()) {
            return;
        }
        this.mqtt.subscribe(channel, this.QOS);
    }

    /**
     * Unsubscribe from a given channel
     * @param channel
     */
    static unsubscribeChannel(channel) {
        console.log('Unsubscribe to channel: ', channel);
        if (!this.mqtt || !this.mqtt.isConnected()) {
            return;
        }
        this.mqtt.unsubscribe(channel);
    }

    /**
     * Publish given payload into given channel
     * @param channel
     * @param payload
     * @returns {boolean}
     */
    static send(channel = null, payload) {
        console.log("Publish data to channel: ", channel, " <-- ", payload);
        if (!this.mqtt || !this.mqtt.isConnected() || !channel || !payload) {
            return;
        }

        this.mqtt.publish(channel, payload, this.QOS, this.RETAIN);
    }

    /**
     * Close the MQTT broker connection
     */
    static close() {
        if (this.mqtt != null) {
            if (this.mqtt.isConnected()) {
                clearTimeout(this.connectionTimer);
                this.mqtt.disconnect();
            }
        }
        this.mqtt = null;
        console.log("MQTT Broker connection is closed!")
    }

    /**
     * Triggered when a message is published
     */
    static onMQTTMessageArrived(message) {
        let topic = message.destinationName;
        let payload = JSON.parse(message.payloadString);

        // handle the message according to it's topic
        switch (topic) {
            case Constants.constants.DISCOVERY_TOPIC:
                console.log("Sensor discovered: ", payload);
                DeviceService.storeDevice(payload);
                break;
            case Constants.constants.DEADVERTISE_TOPIC:
                // find the deadvertised device in react redux
                for (let i = 0; i < store.getState().device.deviceList.length; i++) {
                    let device = store.getState().device.deviceList[i];
                    if (device.parentObjectId === payload.objectId ||
                        device.objectId === payload.objectId) {
                        console.log("Device de-advertised: ", device.name, " - ", device.objectId);
                        // remove device and all it's observation from react-redux
                        store.dispatch({type: "REMOVE_DEVICE", device: device});
                        store.dispatch({type: "REMOVE_DEVICE_OBSERVATIONS", deviceId: device.objectId});

                        store.getState().response.responseList.forEach(response => {
                            Object.keys(response.grouping).forEach(requestId => {
                                Object.keys(response.grouping[requestId]).forEach(aggregationId => {
                                    response.grouping[requestId][aggregationId].forEach(sensorId => {
                                        if (device.objectId === sensorId) {
                                            store.dispatch({
                                                type: 'REMOVE_SENSOR_FROM_RESPONSE',
                                                response: response,
                                                sensorId: sensorId
                                            });

                                            if (response.grouping[requestId][aggregationId].length === 0) {
                                                delete response.grouping[requestId][aggregationId];
                                                if (Object.keys(response.grouping[requestId]).length === 0) {
                                                    delete response.grouping[requestId];
                                                    if (Object.keys(response.grouping).length === 0) {
                                                        store.dispatch({type: "REMOVE_RESPONSE", response: response});
                                                    }
                                                }
                                            }
                                        }
                                    });
                                });
                            });
                        });
                    }
                }
                break;
            case Constants.constants.RESPONSE_TOPIC:
                // save response into react redux
                AspectService.storeResponse(payload, store.getState().request.requestList);
                break;
            case Constants.constants.OBSERVATION_TOPIC:
                // save observation into react redux
                let requestList = store.getState().request.requestList;
                let requestExists = false;

                for (let request of requestList) {
                    if (request.uuid === payload.requestId) {
                        requestExists = true;
                        break;
                    }
                }

                if (!requestExists) {
                    return;
                }

                payload.results = Math.round(payload.results * 100) / 100;
                store.dispatch({type: "ADD_OBSERVATION", observation: payload})

                requestList.forEach(request => {
                    // check if the observation value exceeds the previously set threshold
                    if (request.uuid === payload.requestId) {
                        if (payload.results > request.threshold) {
                            console.log("Threshold exceeded! Value: ",
                                payload.results, ", Threshold: ", request.threshold);
                            // create push notification if it exceeds
                            PushNotification.localNotification({
                                channelId: "threshold", // (required) channelId, if the channel doesn't exist, it will be created with options passed above (importance, vibration, sound). Once the channel is created, the channel will not be update. Make sure your channelId is different if you change these options. If you have created a custom channel, it will apply options of the channel.
                                vibration: 300, // vibration length in milliseconds, ignored if vibrate=false, default: 1000
                                actions: ["Ok", "Open app"], // (Android only) See the doc for notification actions to know more
                                invokeApp: false,
                                title: "Threshold exceeded!", // (optional)
                                message: "Value: " + payload.results + ", Threshold: " + request.threshold, // (required)
                            });
                        }
                    }
                });
                break;
            case Constants.constants.DISCOVERY_ALL_RESPONSE_TOPIC:
                DeviceService.storeDevice(payload);
                break;
            case Constants.constants.OBSERVATION_REQUEST_TOPIC:
                if (!payload.hasOwnProperty("isExternal")) {
                    payload.isExternal = true;
                    store.dispatch({type: "ADD_REQUEST", request: payload});
                }
        }
    }

    /**
     * Runs when a broker connection is lost with a failure
     */
    static onMQTTLost(res) {
        if (this.mqtt !== null && this.mqtt.isConnected()) {
            return;
        }

        console.log('Connection to a broker is lost: ', JSON.stringify(res));
        console.log('Trying to reconnect in 10 seconds ...');
        this.connectionTimer = setTimeout(() => {
            MQTTClientService.connect();
        }, 10000)
    }

    /**
     * Runs when the app successfully connected to a broker
     * @param test
     */
    static onMQTTConnect(test) {
        console.log('Connected to broker.');
        this.subscribeChannel(Constants.constants.DISCOVERY_TOPIC);
        this.subscribeChannel(Constants.constants.DEADVERTISE_TOPIC);
        this.subscribeChannel(Constants.constants.RESPONSE_TOPIC);
        this.subscribeChannel(Constants.constants.OBSERVATION_TOPIC);
        this.subscribeChannel(Constants.constants.DISCOVERY_ALL_RESPONSE_TOPIC);
        this.subscribeChannel(Constants.constants.OBSERVATION_REQUEST_TOPIC);
        this.send(Constants.constants.DISCOVERY_ALL_TOPIC, "{}");
    }
}

