/**
 * Copyright (C) 2021 fortiss GmbH
 * @author Erkan Karabulut – "karabulut@fortiss.org"
 * @version 1.2
 * This file includes functions related with measurement units and types
 */

/**
 * Includes pre-defined measurement units, domains and monitoring aspects
 */
export default class MeasurementUnitService {
    /**
     * Get unit of measurement by the key of a measurement aspect
     * @param aspect
     * @returns {string}
     */
    static getUnitOfMeasurementByAspect(aspect) {
        let unitOfMeasurement;
        switch (aspect) {
            case "temperature":
                unitOfMeasurement = "Celcius";
                break;
            case "energyConsumption":
                unitOfMeasurement = "kWh";
                break;
            default:
                unitOfMeasurement = "unit";
                break;
        }

        return unitOfMeasurement;
    }

    /**
     * Get a map of measurement aspects!?
     * @param messages
     * @returns {Map<any, any>}
     */
    static getSensorMap(messages) {
        let sensorMap = new Map();
        sensorMap.set("movement", messages.movement);
        sensorMap.set("velocity", messages.velocity);
        sensorMap.set("inertia", messages.inertia);
        sensorMap.set("vibration", messages.vibration);
        sensorMap.set("acceleration", messages.acceleration);
        sensorMap.set("occupancy", messages.occupancy);
        sensorMap.set("rotation", messages.rotation);
        sensorMap.set("destination", messages.destination);
        sensorMap.set("orientation", messages.orientation);
        sensorMap.set("inclination", messages.inclination);
        sensorMap.set("proximity", messages.proximity);
        sensorMap.set("presence", messages.presence);
        sensorMap.set("location", messages.location);
        sensorMap.set("layout", messages.layout);
        sensorMap.set("distance", messages.distance);
        sensorMap.set("level", messages.level);
        sensorMap.set("energyConsumption", messages.energyConsumption);
        sensorMap.set("energyProduction", messages.energyProduction);
        sensorMap.set("energyStorage", messages.energyStorage);
        sensorMap.set("temperature", messages.temperature);
        sensorMap.set("humidity", messages.humidity);
        sensorMap.set("luminance", messages.luminance + "/" + messages.light + "/" + messages.optical);
        sensorMap.set("acoustic", messages.acoustic);
        sensorMap.set("radiation", messages.radiation);
        sensorMap.set("gas", messages.gas);
        sensorMap.set("magneticField", messages.magneticField);
        sensorMap.set("chemical", messages.chemical);
        sensorMap.set("electrical", messages.electrical);
        sensorMap.set("color", messages.color);
        sensorMap.set("machineVision", messages.machineVision);
        sensorMap.set("emf", messages.emf);
        sensorMap.set("volume", messages.volume);
        sensorMap.set("pressure", messages.pressure);
        sensorMap.set("density", messages.density);
        sensorMap.set("deformation", messages.deformation);
        sensorMap.set("viscosity", messages.viscosity);
        sensorMap.set("flow", messages.flow);
        sensorMap.set("loadForce", messages.loadForce);
        sensorMap.set("shock", messages.shock);
        sensorMap.set("moisture", messages.moisture);
        sensorMap.set("contact", messages.contact);
        sensorMap.set("strain", messages.strain);
        sensorMap.set("corrosion", messages.corrosion);
        sensorMap.set("electricalConductivity", messages.electricalConductivity);
        sensorMap.set("oxygen", messages.oxygen);
        sensorMap.set("brainElectricalActivity", messages.brainElectricalActivity);
        sensorMap.set("eyeMovement", messages.eyeMovement);
        sensorMap.set("heartElectricalActivity", messages.heartElectricalActivity);
        sensorMap.set("heartRate", messages.heartRate);
        sensorMap.set("respiratoryFlow", messages.respiratoryFlow);
        sensorMap.set("imaging", messages.imaging);
        sensorMap.set("height", messages.height);
        sensorMap.set("weight", messages.weight);
        sensorMap.set("fat", messages.fat);
        sensorMap.set("muscleMass", messages.muscleMass);
        sensorMap.set("boneDensity", messages.boneDensity);
        sensorMap.set("groundReactionForce", messages.groundReactionForce);
        sensorMap.set("gait", messages.gait);
        sensorMap.set("jointTorque", messages.jointTorque);
        sensorMap.set("posture", messages.posture);
        sensorMap.set("skeletalMusclesActivity", messages.skeletalMusclesActivity);
        sensorMap.set("temperature", messages.temperature);
        sensorMap.set("glucoseLevel", messages.glucoseLevel);
        sensorMap.set("ph", messages.ph);
        sensorMap.set("bloodPressure", messages.bloodPressure);
        sensorMap.set("spo2", messages.spo2);
        sensorMap.set("sodiumConductivity", messages.sodiumConductivity);
        sensorMap.set("bloodVesselStiffness", messages.bloodVesselStiffness);
        sensorMap.set("microbialStructure", messages.microbialStructure);
        sensorMap.set("cellStructure", messages.cellStructure);
        sensorMap.set("particulate", "Particulate");

        return sensorMap;
    }

    /**
     * Get a map of monitoring aspects by type
     * @param monitoringAspect
     * @param messages
     * @returns {Map<any, any>}
     */
    static getMonitoringAspectMapByType(monitoringAspect, messages) {
        let monitoringAspectMap = new Map();
        switch (monitoringAspect) {
            case "objectsThings":
                monitoringAspectMap.set("human", messages.human);
                monitoringAspectMap.set("animal", messages.animal);
                monitoringAspectMap.set("plant", messages.plant);
                monitoringAspectMap.set("goods", messages.goods);
                monitoringAspectMap.set("electricalDevices", messages.electricalDevices);
                monitoringAspectMap.set("transport", messages.transport);
                break;
            case "places":
                monitoringAspectMap.set("infrastructure", messages.infrastructure);
                monitoringAspectMap.set("space", messages.space);
                monitoringAspectMap.set("energy", messages.energy);
                monitoringAspectMap.set("environment", messages.environment);
                break;
        }

        return monitoringAspectMap;
    }

    /**
     * Get a map of sub-domains by a more general domain
     * @param domain
     * @param messages
     * @returns {Map<any, any>}
     */
    static getDomainMapByType(domain, messages) {
        let domainMapType = new Map();
        switch (domain) {
            case "health":
                domainMapType.set("hospital", messages.hospital);
                domainMapType.set("nursingHomes", messages.nursingHomes);
                domainMapType.set("privateHomes", messages.privateHomes);
                break;
            case "logistics":
                domainMapType.set("warehouses", messages.warehouses);
                domainMapType.set("transport", messages.transport);
                break;
            case "agriculture":
                domainMapType.set("irrigation", messages.irrigation);
                domainMapType.set("livestock", messages.livestock);
                domainMapType.set("crop", messages.crop);
                break;
            case "smartCity":
                domainMapType.set("publicBuilding", messages.publicBuilding);
                domainMapType.set("mobility", messages.mobility);
                domainMapType.set("greenSpace", messages.greenSpace);
                domainMapType.set("environment", messages.environment);
                domainMapType.set("energy", messages.energy);
                break;
            case "buildingManagement":
                domainMapType.set("hospitality", messages.hospitality);
                domainMapType.set("privateHomes", messages.privateHomes);
                domainMapType.set("offices", messages.offices);
                domainMapType.set("retail", messages.retail);
                break;
            case "manufacturing":
                domainMapType.set("facility", messages.facility);
                domainMapType.set("equipment", messages.equipment);
                break;
        }

        return domainMapType;
    }
}
