/**
 * Copyright (C) 2021 fortiss GmbH
 * @author Jorge de Lima Tostes – "tostes@fortiss.org"
 * @version 1.2
 * Get an xml file by url, parse it and extract service requests
 */

const xml2js = require('xml2js');
import * as config from '../config/ServiceRequestParserConfig';

import uuid from 'react-native-uuid';
import * as RNFS from "react-native-fs";


let requestObject = {
    "featureOfInterest": "",
    "unitOfMeasurement": "unit",
    "threshold": "30",
    "creatorId": "FORTISS_IIOTLAB_DESCRIPTION_PARSER",
    "location": "Per-room",
    "uuid": uuid.v4(),
    "domain": "",
    "domainDetail": "",
    "monitoringAspect": "",
    "monitoringAspectDetail": ""
};

export default class ServiceRequestParser {
    static async getRequestsFromFile(file) {
        let parsedObject = file.content;
        let fileName = file.fileName;
        let fileNameComponents = fileName.split('.');

        const validExtensions = ['xml', 'wsdl', 'owl'];
        const extension = fileNameComponents[fileNameComponents.length - 1];
        const extIsInvalid = (validExtensions.indexOf(extension) === -1);

        if (extIsInvalid) {
            console.error('Invalid file extension')
        } else {
            const wsdlExtensions = ['xml', 'wsdl'];
            if (wsdlExtensions.indexOf(extension) !== -1) {
                let [firstKey] = Object.keys(parsedObject);
                const fileAttributes = parsedObject[firstKey].$;

                const wsdl1 = (fileAttributes['xmlns'] === 'http://schemas.xmlsoap.org/wsdl/');
                const wsdl1Prefix = (fileAttributes['xmlns:wsdl'] === 'http://schemas.xmlsoap.org/wsdl/' && fileAttributes['xmlns:wsdl'] !== fileAttributes['xmlns']);
                const wsdl2 = (fileAttributes['xmlns'] === 'http://www.w3.org/ns/wsdl');
                const wsdl2Prefix = (fileAttributes['xmlns:wsdl'] === 'http://www.w3.org/ns/wsdl' && fileAttributes['xmlns:wsdl'] !== fileAttributes['xmlns']);

                let version;

                if (wsdl1) {
                    version = 'wsdl1';
                    return ServiceRequestParser.wsdl1Parsing(version, parsedObject);
                } else if (wsdl1Prefix) {
                    version = 'wsdl1Prefix';
                    return ServiceRequestParser.wsdl1Parsing(version, parsedObject);
                } else if (wsdl2) {
                    version = 'wsdl2';
                    return ServiceRequestParser.wsdl2Parsing(version, parsedObject);
                } else if (wsdl2Prefix) {
                    version = 'wsdl2Prefix';
                    return ServiceRequestParser.wsdl2Parsing(version, parsedObject);
                } else {
                    return {error: 'Not a valid WSDL file'}
                }
            } else if (extension === 'owl') {
                return ServiceRequestParser.owlParsing(parsedObject);
            }

        }
    }

    static async getFileFromURL(url) {
        const parser = new xml2js.Parser();
        let res;
        try {
            res = await fetch(url)
                .then((response) => response.json())
                .then((json) => {
                    return json;
                });
        } catch (err) {
            console.error("Error: ", err);
            return err;
        }

        let data;
        try {
            data = await parser.parseStringPromise(res.content);
        } catch (err) {
            return err;
        }

        return {
            fileName: res.fileName,
            content: data
        };
    }

    static async titleCase(str) {
        let splitStr = str.toLowerCase().split(' ');
        for (let i = 0; i < splitStr.length; i++) {
            // You do not need to check if i is larger than splitStr length, as your for does that for you
            // Assign it back to the array
            splitStr[i] = splitStr[i].charAt(0).toUpperCase() + splitStr[i].substring(1);
        }
        // Directly return the joined string
        return splitStr.join(' ');
    }


    static async wsdl1Parsing(version, parsedObject) {
        let prefix = '';
        let requests = [];
        let inputs = [];

        if (version === 'wsdl1Prefix') {
            prefix = 'wsdl:'
        }

        for (let i = 0; i < parsedObject[`${prefix}definitions`][`${prefix}portType`][0][`${prefix}operation`][0][`${prefix}input`].length; i++) {
            let input = [];

            let operationInput = parsedObject[`${prefix}definitions`][`${prefix}portType`][0][`${prefix}operation`][0][`${prefix}input`][i].$.message;
            let noprefixOperationInput = operationInput.split(':')[1];

            let resultMessage = parsedObject[`${prefix}definitions`][`${prefix}message`].filter(obj => {
                return obj.$.name === noprefixOperationInput;
            })

            for (let j = 0; j < resultMessage[0][`${prefix}part`].length; j++) {
                let messageInput = resultMessage[0][`${prefix}part`][j].$.element;
                let noprefixMessageInput = messageInput.split(':')[1];

                let resultElement = parsedObject[`${prefix}definitions`][`${prefix}types`][0]['xsd:schema'][0]['xsd:element'].filter(obj => {
                    return obj.$.name === noprefixMessageInput;
                })

                let ontology = resultElement[0]['xsd:complexType'][0]['xsd:sequence'][0]['xsd:element'].filter(obj => {
                    return obj.$.ref === 'ontology';
                })[0]['xsd:element'];
                let QoS = resultElement[0]['xsd:complexType'][0]['xsd:sequence'][0]['xsd:element'].filter(obj => {
                    return obj.$.ref === 'QoS';
                })[0]['xsd:element'];

                for (let k = 0; k < ontology.length; k++) {
                    if (Object.keys(ontology[j]['xsd:element'][0]['xsd:element'][0].$).includes('name')) {
                        input.push([ontology[j].$.ref, ontology[j]['xsd:element'][0].$.ref, ontology[j]['xsd:element'][0]['xsd:element'][0].$.name])
                    } else {
                        input.push([ontology[j].$.ref, ontology[j]['xsd:element'][0].$.ref, ontology[j]['xsd:element'][0]['xsd:element'][0]['xsd:element'][0].$.name])
                    }
                }

                inputs.push(input)
            }
        }

        let error = ``;

        for (let i = 0; i < inputs.length; i++) {
            let request = {
                "featureOfInterest": "",
                "unitOfMeasurement": "unit",
                "threshold": "30",
                "creatorId": "FORTISS_IIOTLAB_DESCRIPTION_PARSER",
                "location": "Per-room",
                "uuid": uuid.v4(),
                "domain": "",
                "domainDetail": "",
                "monitoringAspect": "",
                "monitoringAspectDetail": ""
            };

            for (let j = 0; j < inputs[i].length; j++) {
                if (inputs[i][j][0].replace(/_|-/g, ' ').toLowerCase().trim() === 'domain') {
                    if (inputs[i][j][1].replace(/_|-/g, ' ').toLowerCase().trim() in config['ontology']['domain']) {
                        request["domain"] = inputs[i][j][1].replace(/_|-/g, ' ').toLowerCase().trim();
                        let element = config['ontology']['domain'][request["domain"]];
                        if (typeof (element[Object.keys(element)[0]]) != 'object') {
                            if (element.includes(inputs[i][j][2].replace(/_|-/g, ' ').toLowerCase().trim())) {
                                request["domainDetail"] = inputs[i][j][2].replace(/_|-/g, ' ').toLowerCase().trim();
                            } else {
                                error += `The Domain Detail used in the input ${i + 1} is not included in the ${request["domain"]} Domain\n`;
                            }
                        } else {
                            for (let k = 0; k < Object.keys(element).length; k++) {
                                if (element[Object.keys(element)[j]].includes(inputs[i][j][2].replace(/_|-/g, ' ').toLowerCase().trim())) {
                                    request["domainDetail"] = inputs[i][j][2].replace(/_|-/g, ' ').toLowerCase().trim();
                                } else {
                                    if (k + 1 === Object.keys(element).length) {
                                        error += `The Domain Detail used in the input ${i + 1} is not included in the ${request["domain"]} Domain\n`;
                                    }
                                }
                            }
                        }
                    } else {
                        error += `The Domain used in the input ${i + 1} is not included in the ontology \n`;
                    }
                } else if (inputs[i][j][0].replace(/_|-/g, ' ').toLowerCase().trim() === 'monitoring aspects') {
                    if (inputs[i][j][1].replace(/_|-/g, ' ').toLowerCase().trim() in config['ontology']['monitoring aspect']) {
                        request["monitoringAspect"] = inputs[i][j][1].replace(/_|-/g, ' ').toLowerCase().trim();
                        let element = config['ontology']['monitoring aspect'][request["monitoringAspect"]];
                        if (typeof (element[Object.keys(element)[0]]) != 'object') {
                            if (element.includes(inputs[i][j][2].replace(/_|-/g, ' ').toLowerCase().trim())) {
                                request["monitoringAspectDetail"] = inputs[i][j][2].replace(/_|-/g, ' ').toLowerCase().trim();
                            } else {
                                error += `The Monitoring Aspect Detail used in the input ${i + 1} is not included in the ${request["monitoringAspect"]} Monitoring Aspect\n`;
                            }
                        } else {
                            for (let k = 0; k < Object.keys(element).length; k++) {
                                if (element[Object.keys(element)[j]].includes(inputs[i][j][2].replace(/_|-/g, ' ').toLowerCase().trim())) {
                                    request["monitoringAspectDetail"] = inputs[i][j][2].replace(/_|-/g, ' ').toLowerCase().trim();
                                } else {
                                    if (k + 1 === Object.keys(element).length) {
                                        error += `The Monitoring Aspect Detail used in the input ${i + 1} is not included in the ${request["monitoringAspect"]} Monitoring Aspect\n`;
                                    }
                                }
                            }
                        }
                    } else {
                        error += `The Monitoring Aspect used in the input ${i + 1} is not included in the ontology \n`;
                    }
                } else if (inputs[i][j][0].replace(/_|-/g, ' ').toLowerCase().trim() === 'measurement types') {
                    if (inputs[i][j][1].replace(/_|-/g, ' ').toLowerCase().trim() in config['ontology']['measurement type']) {
                        let element = config['ontology']['measurement type'][inputs[i][j][1].replace(/_|-/g, ' ').toLowerCase().trim()];
                        if (typeof (element[Object.keys(element)[0]]) != 'object') {
                            if (element.includes(inputs[i][j][2].replace(/_|-/g, ' ').toLowerCase().trim())) {
                                request["featureOfInterest"] = inputs[i][j][2].replace(/_|-/g, ' ').toLowerCase().trim();
                            } else {
                                error += `The Feature of Interest used in the input ${i + 1} is not included in the ${inputs[i][j][1].replace(/_|-/g, ' ')} Measurement Type\n`;
                            }
                        } else {
                            for (let k = 0; k < Object.keys(element).length; k++) {
                                if (element[Object.keys(element)[j]].includes(inputs[i][j][2].replace(/_|-/g, ' ').toLowerCase().trim())) {
                                    request["featureOfInterest"] = inputs[i][j][2].replace(/_|-/g, ' ').toLowerCase().trim();
                                } else {
                                    if (k + 1 === Object.keys(element).length) {
                                        error += `The Feature of Interest used in the input ${i + 1} is not included in the ${measurementType.split(':')[0].replace(/_|-/g, ' ')} Measurement Type\n`;
                                    }
                                }
                            }
                        }
                    } else {
                        error += `The Measurement Type used in the input ${i + 1} is not included in the ontology \n`;
                    }
                }
            }
            request['domain'] = (await ServiceRequestParser.titleCase(request['domain']));
            request['domainDetail'] = (await ServiceRequestParser.titleCase(request['domainDetail']));
            request['monitoringAspect'] = (await ServiceRequestParser.titleCase(request['monitoringAspect']));
            request['monitoringAspectDetail'] = (await ServiceRequestParser.titleCase(request['monitoringAspectDetail']));
            request['featureOfInterest'] = (await ServiceRequestParser.titleCase(request['featureOfInterest']));

            requests.push(request);
        }

        if (error.length > 0) {
            return error;
        } else {
            return requests;
        }
    }


    static async wsdl2Parsing(version, parsedObject) {
        let prefix = '';
        let requests = [];
        let inputs = [];

        if (version === 'wsdl2Prefix') {
            prefix = 'wsdl:'
        }

        for (let i = 0; i < parsedObject[`${prefix}description`][`${prefix}interface`][0][`${prefix}operation`][0][`${prefix}input`].length; i++) {
            let input = [];

            let operationInput = parsedObject[`${prefix}description`][`${prefix}interface`][0][`${prefix}operation`][0][`${prefix}input`][i].$.element;
            let noprefixOperationInput = operationInput.split(':')[1];

            let resultElement = parsedObject[`${prefix}description`][`${prefix}types`][0]['xsd:schema'][0]['xsd:element'].filter(obj => {
                return obj.$.name === noprefixOperationInput;
            })

            let ontology = resultElement[0]['xsd:complexType'][0]['xsd:sequence'][0]['xsd:element'].filter(obj => {
                return obj.$.ref === 'Ontology';
            })[0]['xsd:element'];
            let QoS = resultElement[0]['xsd:complexType'][0]['xsd:sequence'][0]['xsd:element'].filter(obj => {
                return obj.$.ref === 'QoS';
            })[0]['xsd:element'];

            for (let j = 0; j < ontology.length; j++) {
                if (Object.keys(ontology[j]['xsd:element'][0]['xsd:element'][0].$).includes('name')) {
                    input.push([ontology[j].$.ref, ontology[j]['xsd:element'][0].$.ref, ontology[j]['xsd:element'][0]['xsd:element'][0].$.name])
                } else {
                    input.push([ontology[j].$.ref, ontology[j]['xsd:element'][0].$.ref, ontology[j]['xsd:element'][0]['xsd:element'][0]['xsd:element'][0].$.name])
                }
            }

            inputs.push(input)
        }

        let error = ``;

        for (let i = 0; i < inputs.length; i++) {
            let request = {
                "featureOfInterest": "",
                "unitOfMeasurement": "unit",
                "threshold": "30",
                "creatorId": "FORTISS_IIOTLAB_DESCRIPTION_PARSER",
                "location": "Per-room",
                "uuid": uuid.v4(),
                "domain": "",
                "domainDetail": "",
                "monitoringAspect": "",
                "monitoringAspectDetail": ""
            };

            for (let j = 0; j < inputs[i].length; j++) {
                if (inputs[i][j][0].replace(/_|-/g, ' ').toLowerCase().trim() === 'domain') {
                    if (inputs[i][j][1].replace(/_|-/g, ' ').toLowerCase().trim() in config['ontology']['domain']) {
                        request["domain"] = inputs[i][j][1].replace(/_|-/g, ' ').toLowerCase().trim();
                        let element = config['ontology']['domain'][request["domain"]];
                        if (typeof (element[Object.keys(element)[0]]) != 'object') {
                            if (element.includes(inputs[i][j][2].replace(/_|-/g, ' ').toLowerCase().trim())) {
                                request["domainDetail"] = inputs[i][j][2].replace(/_|-/g, ' ').toLowerCase().trim();
                            } else {
                                error += `The Domain Detail used in the input ${i + 1} is not included in the ${request["domain"]} Domain\n`;
                            }
                        } else {
                            for (let k = 0; k < Object.keys(element).length; k++) {
                                if (element[Object.keys(element)[j]].includes(inputs[i][j][2].replace(/_|-/g, ' ').toLowerCase().trim())) {
                                    request["domainDetail"] = inputs[i][j][2].replace(/_|-/g, ' ').toLowerCase().trim();
                                } else {
                                    if (k + 1 === Object.keys(element).length) {
                                        error += `The Domain Detail used in the input ${i + 1} is not included in the ${request["domain"]} Domain\n`;
                                    }
                                }
                            }
                        }
                    } else {
                        error += `The Domain used in the input ${i + 1} is not included in the ontology \n`;
                    }
                } else if (inputs[i][j][0].replace(/_|-/g, ' ').toLowerCase().trim() === 'monitoring aspects') {
                    if (inputs[i][j][1].replace(/_|-/g, ' ').toLowerCase().trim() in config['ontology']['monitoring aspect']) {
                        request["monitoringAspect"] = inputs[i][j][1].replace(/_|-/g, ' ').toLowerCase().trim();
                        let element = config['ontology']['monitoring aspect'][request["monitoringAspect"]];
                        if (typeof (element[Object.keys(element)[0]]) != 'object') {
                            if (element.includes(inputs[i][j][2].replace(/_|-/g, ' ').toLowerCase().trim())) {
                                request["monitoringAspectDetail"] = inputs[i][j][2].replace(/_|-/g, ' ').toLowerCase().trim();
                            } else {
                                error += `The Monitoring Aspect Detail used in the input ${i + 1} is not included in the ${request["monitoringAspect"]} Monitoring Aspect\n`;
                            }
                        } else {
                            for (let k = 0; k < Object.keys(element).length; k++) {
                                if (element[Object.keys(element)[j]].includes(inputs[i][j][2].replace(/_|-/g, ' ').toLowerCase().trim())) {
                                    request["monitoringAspectDetail"] = inputs[i][j][2].replace(/_|-/g, ' ').toLowerCase().trim();
                                } else {
                                    if (k + 1 === Object.keys(element).length) {
                                        error += `The Monitoring Aspect Detail used in the input ${i + 1} is not included in the ${request["monitoringAspect"]} Monitoring Aspect\n`;
                                    }
                                }
                            }
                        }
                    } else {
                        error += `The Monitoring Aspect used in the input ${i + 1} is not included in the ontology \n`;
                    }
                } else if (inputs[i][j][0].replace(/_|-/g, ' ').toLowerCase().trim() === 'measurement types') {
                    if (inputs[i][j][1].replace(/_|-/g, ' ').toLowerCase().trim() in config['ontology']['measurement type']) {
                        let element = config['ontology']['measurement type'][inputs[i][j][1].replace(/_|-/g, ' ').toLowerCase().trim()];
                        if (typeof (element[Object.keys(element)[0]]) != 'object') {
                            if (element.includes(inputs[i][j][2].replace(/_|-/g, ' ').toLowerCase().trim())) {
                                request["featureOfInterest"] = inputs[i][j][2].replace(/_|-/g, ' ').toLowerCase().trim();
                            } else {
                                error += `The Feature of Interest used in the input ${i + 1} is not included in the ${inputs[i][j][1].replace(/_|-/g, ' ')} Measurement Type\n`;
                            }
                        } else {
                            for (let k = 0; k < Object.keys(element).length; k++) {
                                if (element[Object.keys(element)[j]].includes(inputs[i][j][2].replace(/_|-/g, ' ').toLowerCase().trim())) {
                                    request["featureOfInterest"] = inputs[i][j][2].replace(/_|-/g, ' ').toLowerCase().trim();
                                } else {
                                    if (k + 1 === Object.keys(element).length) {
                                        error += `The Feature of Interest used in the input ${i + 1} is not included in the ${measurementType.split(':')[0].replace(/_|-/g, ' ')} Measurement Type\n`;
                                    }
                                }
                            }
                        }
                    } else {
                        error += `The Measurement Type used in the input ${i + 1} is not included in the ontology \n`;
                    }
                }
            }
            request['domain'] = (await ServiceRequestParser.titleCase(request['domain']));
            request['domainDetail'] = (await ServiceRequestParser.titleCase(request['domainDetail']));
            request['monitoringAspect'] = (await ServiceRequestParser.titleCase(request['monitoringAspect']));
            request['monitoringAspectDetail'] = (await ServiceRequestParser.titleCase(request['monitoringAspectDetail']));
            request['featureOfInterest'] = (await ServiceRequestParser.titleCase(request['featureOfInterest']));

            requests.push(request);
        }

        if (error.length > 0) {
            return error;
        } else {
            return requests;
        }
    }

    static async owlParsing(parsedObject) {
        //Parsing Process
        let xmlns = parsedObject[Object.keys(parsedObject)].$['xmlns'];

        let ontologyPrefix = parsedObject[Object.keys(parsedObject)].$['xmlns:Ontology'];

        let matched = Object.keys(parsedObject[Object.keys(parsedObject)].$).filter(function (key) {
            return parsedObject[Object.keys(parsedObject)].$[key] === xmlns;
        });

        let serviceAttribute = matched.filter(obj => {
            return obj !== 'xmlns';
        });

        let serviceName = serviceAttribute[0].split(':')[1];

        let atomicProcess = parsedObject[Object.keys(parsedObject)]['owl:NamedIndividual'].filter(key => {
            if (Object.keys(key).includes('rdf:type') && Object.keys(key['rdf:type'][0]).includes('$')) {
                return key['rdf:type'][0].$['rdf:resource'].split('#')[1] === 'AtomicProcess';
            }
        });

        let inputNames = [];

        for (let i = 0; i < atomicProcess[0]['Process:hasInput'].length; i++) {
            let inputName = atomicProcess[0]['Process:hasInput'][i].$['rdf:resource'].split('#')[1];

            inputNames.push(inputName);
        }

        let inputs = [];
        let error = ``;

        //Validation Process
        for (let i = 0; i < inputNames.length; i++) {
            let input = requestObject;

            let inputIndividual = parsedObject[Object.keys(parsedObject)]['owl:NamedIndividual'].filter(obj => {
                return obj.$['rdf:about'].split('#')[1] === inputNames[i];
            });
            let inputPrefix = ''

            if (typeof (inputIndividual[0][`${serviceName}:hasDomain`]) !== 'undefined') {
                inputPrefix = `${serviceName}:`;
            } else if (typeof (inputIndividual[0][`hasDomain`]) !== 'undefined') {

            } else if (ontologyPrefix === "http://www.semanticweb.org/tostes/ontologies/2021/6/Ontology#") {
                inputPrefix = 'Ontology:'
            } else {
                return 'Unknown Prefix for input inside OWL file'
            }

            let domain = inputIndividual[0][`${inputPrefix}hasDomain`][0].$['rdf:resource'].split('#')[1];
            if (domain.split(':')[0].replace(/_|-/g, ' ').toLowerCase().trim() in config['default']["ontology"]['domain']) {
                input["domain"] = domain.split(':')[0].replace(/_|-/g, ' ').toLowerCase().trim();
                let element = config['default']["ontology"]['domain'][input["domain"]];

                if (typeof (element[Object.keys(element)[0]]) != 'object') {
                    if (element.includes(domain.split(':')[1].replace(/_|-/g, ' ').toLowerCase().trim())) {
                        input["domainDetail"] = domain.split(':')[1].replace(/_|-/g, ' ').toLowerCase().trim();
                    } else {
                        error += `The Domain Detail used in the input ${i + 1} is not included in the ${input["domain"]} Domain\n`;
                    }
                } else {
                    for (let j = 0; j < Object.keys(element).length; j++) {
                        if (element[Object.keys(element)[j]].includes(domain.split(':')[1].replace(/_|-/g, ' ').toLowerCase().trim())) {
                            input["domainDetail"] = domain.split(':')[1].replace(/_|-/g, ' ').toLowerCase().trim();
                        } else {
                            if (j + 1 === Object.keys(element).length) {
                                error += `The Domain Detail used in the input ${i + 1} is not included in the ${input["domain"]} Domain\n`;
                            }
                        }
                    }
                }
            } else {
                error += `The Domain used in the input ${i + 1} is not included in the Ontology \n`;
            }

            let measurementType = inputIndividual[0][`${inputPrefix}hasMeasurementType`][0].$['rdf:resource'].split('#')[1];
            if (measurementType.split(':')[0].replace(/_|-/g, ' ').toLowerCase().trim() in config['default']["ontology"]['measurement type']) {
                let element = config['default']["ontology"]['measurement type'][measurementType.split(':')[0].replace(/_|-/g, ' ').toLowerCase().trim()];

                if (typeof (element[Object.keys(element)[0]]) != 'object') {
                    if (element.includes(measurementType.split(':')[1].replace(/_|-/g, ' ').toLowerCase().trim())) {
                        input["featureOfInterest"] = measurementType.split(':')[1].replace(/_|-/g, ' ').toLowerCase().trim();
                    } else {
                        error += `The Feature of Interest used in the input ${i + 1} is not included in the ${measurementType.split(':')[0].replace(/_|-/g, ' ')} Measurement Type\n`;
                    }
                } else {
                    for (let j = 0; j < Object.keys(element).length; j++) {
                        if (element[Object.keys(element)[j]].includes(measurementType.split(':')[1].replace(/_|-/g, ' ').toLowerCase().trim())) {
                            input["featureOfInterest"] = measurementType.split(':')[1].replace(/_|-/g, ' ').toLowerCase().trim();
                        } else {
                            if (j + 1 === Object.keys(element).length) {
                                error += `The Feature of Interest used in the input ${i + 1} is not included in the ${measurementType.split(':')[0].replace(/_|-/g, ' ')} Measurement Type\n`;
                            }
                        }
                    }
                }
            } else {
                error += `The Measurement Type used in the input ${i + 1} is not included in the Ontology \n`;
            }

            let monitoringAspect = inputIndividual[0][`${inputPrefix}hasMonitoringAspect`][0].$['rdf:resource'].split('#')[1];
            if (monitoringAspect.split(':')[0].replace(/_|-/g, ' ').toLowerCase().trim() in config['default']["ontology"]['monitoring aspect']) {
                input["monitoringAspect"] = monitoringAspect.split(':')[0].replace(/_|-/g, ' ').toLowerCase().trim();
                let element = config['default']["ontology"]['monitoring aspect'][input["monitoringAspect"]];

                if (typeof (element[Object.keys(element)[0]]) != 'object') {
                    if (element.includes(monitoringAspect.split(':')[1].replace(/_|-/g, ' ').toLowerCase().trim())) {
                        input["monitoringAspectDetail"] = monitoringAspect.split(':')[1].replace(/_|-/g, ' ').toLowerCase().trim();
                    } else {
                        error += `The Monitoring Aspect Detail used in the input ${i + 1} is not included in the ${input["monitoringAspect"]} Monitoring Aspect\n`;
                    }
                } else {
                    for (let j = 0; j < Object.keys(element).length; j++) {
                        if (element[Object.keys(element)[j]].includes(monitoringAspect.split(':')[1].replace(/_|-/g, ' ').toLowerCase().trim())) {
                            input["monitoringAspectDetail"] = monitoringAspect.split(':')[1].replace(/_|-/g, ' ').toLowerCase().trim();
                        } else {
                            if (j + 1 === Object.keys(element).length) {
                                error += `The Monitoring Aspect Detail used in the input ${i + 1} is not included in the ${input["monitoringAspect"]} Monitoring Aspect\n`;
                            }
                        }
                    }
                }
            } else {
                error += `The Monitoring Aspect used in the input ${i + 1} is not included in the Ontology \n`;
            }

            input['domain'] = (await ServiceRequestParser.titleCase(input['domain']));
            input['domainDetail'] = (await ServiceRequestParser.titleCase(input['domainDetail']));
            input['monitoringAspect'] = (await ServiceRequestParser.titleCase(input['monitoringAspect']));
            input['monitoringAspectDetail'] = (await ServiceRequestParser.titleCase(input['monitoringAspectDetail']));
            input['featureOfInterest'] = (await ServiceRequestParser.titleCase(input['featureOfInterest']));

            inputs.push(input);
        }

        if (error.length > 0) {
            return error;
        } else {
            return inputs;
        }
    }
}
