/**
* MIT License
*
* Copyright (c) 2020 fortiss GmbH
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
* 
* SPDX-License-Identifier: MIT
* @author Nisrine Bnouhanna 
* @version 1.0
* creates coaty container for the TSMatch Engine
* Uses coatyio Siemens AG. Licensed under the MIT License
*/

import {Components, Configuration, Container, CoreTypes, mergeConfigurations, Runtime, User} from "@coaty/core";
import {NodeUtils} from "@coaty/core/runtime-node";

import {serviceConfig} from "../shared/config";

import {agentInfo} from "./agent.info";
import {FeatureOfInterestController} from "./FeatureOfInterestController";
import {SensorController} from "./SensorController";
import {BaseRepository} from "../shared/repository/BaseRepository";


const components: Components = {
    controllers: {
        FeatureOfInterestController,
        SensorController: SensorController
    },
};

const configuration: Configuration = mergeConfigurations(
    serviceConfig(agentInfo, "Service"),
    {
        common: {
            extra: {
                // The service user that acts as creator for task requests.
                serviceUser: {
                    objectId: Runtime.newUuid(),
                    objectType: CoreTypes.OBJECT_TYPE_USER,
                    coreType: "User",
                    name: "IoT App",
                    names: {
                        givenName: "IIoT group",
                        familyName: "IoT Lab",
                    },
                } as User,
            },
        },
        controllers: {
            FeatureOfInterestController: {
                shouldAdvertiseSnapshots: false,
                shouldPersistLocalSnapshots: true,
                shouldPersistObservedSnapshots: false,
                shouldReplyToQueries: true,
            },
            SensorController: {
                requestGenerationInterval: 10000,
            },
        },
    });


NodeUtils.handleProcessTermination();

// Register database adapters before resolving a Coaty container so that
// controllers can create database contexts on initialization.

BaseRepository.registerDBAdapter();

// First, initialize the database
BaseRepository.initializeDatabase(configuration.databases)
    .then(() => {
        // Then, create the Coaty container with the specified components and
        // autostart the communication manager.
        const container = Container.resolve(components, configuration);

        // Log broker connection state changes (online/offline) to the console.
        NodeUtils.logCommunicationState(container);
    })
    .catch(error => NodeUtils.logError(error, "Failed to initialize database."));
