/**
* MIT License
*
* Copyright (c) 2020 fortiss GmbH
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
* 
* SPDX-License-Identifier: MIT
* @author Nisrine Bnouhanna 
* @version 1.0
 * creates feature of Interest controller for the TSMatch Engine
 * Uses coatyio Siemens AG. Licensed under the MIT License
 */

import {CoatyObject, DiscoverEvent, ObjectCacheController, Uuid} from "@coaty/core";
import {DbContext} from "@coaty/core/db";
import {FeatureOfInterest, SensorThingsTypes} from "@coaty/core/sensor-things";
import {Observable} from "rxjs";
import {timeout} from "rxjs/operators";
import {FeatureOfInterestEventHandler} from "../shared/eventhandler/FeatureOfInterestEventHandler";

/**
 * Discover feature of interest objects related to a specific sensor thing
 */
export class FeatureOfInterestController extends ObjectCacheController<CoatyObject> {
    private _dbCtx: DbContext;

    onInit() {
        super.onInit();
        this._dbCtx = new DbContext(this.runtime.databaseOptions["neo4j"]);
    }

    /**
     * Resolves the sensor thing object for the given object ID. Resolution
     * times out after 5 seconds.
     *
     * @param objectId the object id of the object to resolve
     */
    resolveSensorThingsObject(objectId: Uuid) {
        return this.resolveObject(objectId)
            .pipe(timeout(5000)) as Observable<FeatureOfInterest>;
    }

    onCommunicationManagerStarting() {
        super.onCommunicationManagerStarting();
        this._observeAdvertiseSensor();
        this._discoverFeatureOfInterest();
    }

    private _observeAdvertiseSensor() {
        return this.communicationManager
            .observeAdvertiseWithObjectType(SensorThingsTypes.OBJECT_TYPE_FEATURE_OF_INTEREST)
            .subscribe(event => {
                const featureOfInterest = event.data.object;
                FeatureOfInterestEventHandler.featureOfInterestAddedHandler(this._dbCtx, featureOfInterest);
            });
    }

    private _discoverFeatureOfInterest() {
        return this.communicationManager.publishDiscover(DiscoverEvent
            .withObjectTypes([SensorThingsTypes.OBJECT_TYPE_FEATURE_OF_INTEREST]))
            .subscribe(event => {
                const featureOfInterest = event.data.object;
                FeatureOfInterestEventHandler.featureOfInterestAddedHandler(this._dbCtx, featureOfInterest);
            });
    }
}
