/**
* MIT License
*
* Copyright (c) 2020 fortiss GmbH
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
* 
* SPDX-License-Identifier: MIT
* @author Nisrine Bnouhanna 
* @version 1.0
* an API that receives service request and publishes observation based on the specified request
* Uses coatyio Siemens AG. Licensed under the MIT License
*/
import {DbContext} from "@coaty/core/db";
import {ThingSensorObservationObserverController} from "@coaty/core/sensor-things";
import {ThingRepository} from "../shared/repository/ThingRepository";
import {LocationRepository} from "../shared/repository/LocationRepository";
import {JSONUtil} from "../shared/util/JSONUtil";
import {SensorEventHandler} from "../shared/eventhandler/SensorEventHandler";

export class SensorController extends ThingSensorObservationObserverController {
    private _dbCtx: DbContext;
    private sensorQueue = [];
    private isWorking = false;

    onInit() {
        super.onInit();
        this._dbCtx = new DbContext(this.runtime.databaseOptions["neo4j"]);
    }

    onCommunicationManagerStarting() {
        super.onCommunicationManagerStarting();
        this.communicationManager
            .observeRaw(process.env.TOPIC_REQUEST_DISCOVERY_ALL)
            .subscribe(topicWithPayload => this._GetAllSensors(topicWithPayload.toString()));

        // Monitor information about changes in the currently registered sensors.
        this.registeredSensorsChangeInfo$.subscribe(changeInfo => {
            if (changeInfo.added.length !== 0) {
                changeInfo.added.forEach(sensor => {
                    SensorEventHandler.sensorAddedEventHandler(this.communicationManager, this.runtime.newUuid(), this._dbCtx, sensor);
                    this.sensorQueue.push(sensor);
                    if (!this.isWorking) {
                        this.isWorking = true;
                        this.observeSensors();
                    }
                });
            } else if (changeInfo.removed.length !== 0) {
                changeInfo.removed.forEach(sensor => {
                    SensorEventHandler.sensorDeletedEventHandler(this.communicationManager, this.runtime.newUuid(), this._dbCtx, sensor);
                    this.communicationManager
                        .publishRaw(process.env.TOPIC_DEADVERTIZE, JSON.stringify(sensor["thing"]));
                });
            }
        });

        this.sensorObservation$.subscribe(({obs, sensor}) => {
            obs["sensorId"] = obs["parentObjectId"];
            delete obs["parentObjectId"];
            this.communicationManager.publishRaw(process.env.TOPIC_RAW_OBSERVATION, JSON.stringify(obs));
        });

    }

    /**
     * Observe newly discovered sensors, get the location/placement of the sensors and
     * merge it with the sensor description and finally advertise the result
     */
    observeSensors() {
        let sensor = this.sensorQueue.pop();
        if (!sensor) {
            this.isWorking = false;
            return;
        }

        sensor = JSONUtil.linearize(sensor);
        // find location of this sensor
        LocationRepository.getFeatureOfInterest(this._dbCtx, sensor["thing_objectId"]).then(result => {
            if (result.length > 0) {
                const location = {
                    room: result[0]._fields[0],
                    floor: result[0]._fields[1],
                    building: {
                        name: result[0]._fields[2],
                        address: result[0]._fields[3],
                    },
                };

                // advertise the sensor and it's location
                const payload = {sensor, location};
                // todo: enrich sensor description by sample ontology/taxonomy data
                this.communicationManager
                    .publishRaw(process.env.TOPIC_DISCOVERY, JSON.stringify(payload));
                this.observeSensors();
            } else {
                this.sensorQueue.push(sensor);
                setTimeout(() => this.observeSensors(), 50);
            }
        });
    }

    /**
     * Get all the registered sensors
     * @param message
     */
    _GetAllSensors(message) {
        ThingRepository.getAllSensors(this._dbCtx).then(result => {
            result.forEach(sensorData => {
                const sensor = sensorData._fields[0].properties;

                // find the location info for each of the sensor
                LocationRepository.getFeatureOfInterest(this._dbCtx, sensor.parentObjectId).then(result => {
                    const location = {
                        room: result[0]._fields[0],
                        floor: result[0]._fields[1],
                        building: {
                            name: result[0]._fields[2],
                            address: result[0]._fields[3],
                        },
                    };

                    // publish the sensors along with their location
                    const payload = {sensor, location};
                    this.communicationManager
                        .publishRaw(process.env.TOPIC_RESPONSE_DISCOVERY_ALL, JSON.stringify(payload));
                });
            });
        });
        console.log("Request for all sensor available received !!");
    }
}
