/**
* MIT License
*
* Copyright (c) 2020 fortiss GmbH
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
* 
* SPDX-License-Identifier: MIT
* @author Nisrine Bnouhanna
 * A Neo4j db adapter for Coaty
 */

import {DbAdapterBase} from "@coaty/core/db";
import {DbConnectionInfo} from "@coaty/core";
import * as neo4j from "neo4j-driver";
import {NodeUtils} from "@coaty/core/runtime-node";

/**
 * Neo4j db adapter for Coaty
 */
export class Neo4jAdapter extends DbAdapterBase {
    public static _driver;
    public connectionInfo: DbConnectionInfo;

    constructor(connectionInfo: DbConnectionInfo) {
        super(connectionInfo);
        this.connectionInfo = connectionInfo;

        Neo4jAdapter._driver = neo4j.driver(
            connectionInfo.connectionString,
            neo4j.auth.basic(
                connectionInfo.connectionOptions.NEO4J_USERNAME,
                connectionInfo.connectionOptions.NEO4J_PASSWORD
            )
        )

        this.registerExtension("initDatabase");
        this.registerExtension("runQuery");
    }

    /**
     * Check if the given NEO4J_DATABASE exists
     * Create it in case it's not exists and create a db user
     * @param dbInfo: database connection information
     */
    async initDatabase(dbInfo: DbConnectionInfo) {
        // check if the given db exists
        let database = await this.runQuery(
            "show databases where name=$db_name", {
                db_name: dbInfo.connectionOptions.NEO4J_DATABASE
            }
        )

        if (database == false) {
            let createDb = this.createDatabase(dbInfo.connectionOptions.NEO4J_DATABASE);
            if (!createDb) {
                NodeUtils.logError("Neo4J database creation is failed! Exiting...");
                process.exit(1);
            }
        }

        let user = await this.runQuery(
            "show users where user=$username", {
                username: dbInfo.connectionOptions.NEO4J_USERNAME
            }
        );

        if (!user) {
            let createUser = this.createUser(dbInfo.connectionOptions.NEO4J_USERNAME, dbInfo.connectionOptions.NEO4J_PASSWORD);
            if (!createUser) {
                NodeUtils.logError("Couldn't create the db user! Exiting...");
                process.exit(1);
            }
        }

        await this.createConstraints();
        await this.createIndexes();

        NodeUtils.logInfo("Database initialization is done!");
    }

    /**
     * Create db constraints
     */
    async createConstraints() {
        // sensor.objectId must be unique
        await this.runQuery(`create constraint constraint_sensor_objectId if not exists
            on (s:Sensor) assert s.objectId is unique`, {});
        // thing.objectId must be unique
        await this.runQuery(`create constraint constraint_thing_objectId if not exists
            on (t:Thing) assert t.objectId is unique`, {});
    }

    async createIndexes() {
        await this.runQuery("CREATE INDEX index_sensor_objectId IF NOT EXISTS FOR (s:Sensor) ON (s.objectId)", {});
        await this.runQuery("CREATE INDEX index_thing_objectId IF NOT EXISTS FOR (t:Thing) ON (t.objectId)", {});
    }

    /**
     * Create a database with the given name
     * @param dbName
     */
    async createDatabase(dbName) {
        let result = await this.runQuery(
            "create database $dbName", {
                dbName: dbName
            }
        )

        if (!result) {
            /* TODO: We don't know what result returns in case of success.
                Because only enterprise version of Neo4j allows creation of a new database
                In case we use the enterprise version, check if the db created successfully
            */
        }

        return true;
    }

    /**
     * Create a user with the given username and password
     * @param dbUser
     * @param dbPassword
     */
    async createUser(dbUser, dbPassword) {
        let result = await this.runQuery(
            "CREATE USER $user SET PASSWORD $password", {
                user: dbUser,
                password: dbPassword
            }
        )

        return result;
    }

    /**
     * Run the given Cypher query with given parameters
     * @param query
     * @param parameters
     */
    async runQuery(query, parameters) {
        if (!query) {
            console.warn("Invalid must not be empty!");
            return false;
        }

        const session = Neo4jAdapter._driver.session({
            database: this.connectionInfo.connectionOptions.NEO4J_DATABASE,
            defaultAccessMode: neo4j.session.WRITE
        });

        let results = null;
        try {
            results = await session.run(
                query,
                parameters
            );
        } catch (error) {
            console.error("An error occurred during query execution: ", error);
        } finally {
            await session.close();
        }

        if (!results || !results.records) {
            console.warn("Result is empty");
            return false;
        }

        return results.records;
    }
}
