/**
* MIT License
*
* Copyright (c) 2020 fortiss GmbH
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
* 
* SPDX-License-Identifier: MIT
* @author Nisrine Bnouhanna 
* @version 1.0
 * creates task controller for the TSMatch Engine
 * Uses coatyio Siemens AG. Licensed under the MIT License
 */

import {DbContext} from "@coaty/core/db";

export class FeatureOfInterestEventHandler {
    static working = false;
    static queue = [];

    /**
     * Add given featureOfInterest object to the queue
     * Objects in the queue will be process by iterateLocationQueue method
     * @param dbContext
     * @param featureOfInterest
     */
    public static featureOfInterestAddedHandler(dbContext: DbContext, featureOfInterest: {}) {
        this.queue.push(featureOfInterest);
        if (!this.working) {
            this.iterateLocationQueue(dbContext);
        }
    }

    /**
     * Check if the location data in the queue exists in the db or not
     * Create new location nodes in case they not already exists
     * Then match existing sensors with the location nodes
     * @param dbContext
     */
    public static iterateLocationQueue(dbContext: DbContext) {
        this.working = true;
        let featureOfInterest = this.queue.pop();
        if (featureOfInterest && featureOfInterest["metadata"]) {
            // check if the building exists
            dbContext.callExtension("runQuery",
                `match (b:Building:Location {name: $buildingName, address: $buildingAddress}) return b`, {
                    buildingAddress: featureOfInterest["metadata"].building.address,
                    buildingName: featureOfInterest["metadata"].building.name
                }).then(result => {
                if (result && result.length > 0) {
                    // check if the floor exists
                    dbContext.callExtension("runQuery",
                        `match (f:Floor:Location {name: $floorName})-[:AT_LOCATION]->(b:Building:Location {name: $buildingName, address: $buildingAddress}) return f`, {
                            floorName: featureOfInterest["metadata"].floor,
                            buildingAddress: featureOfInterest["metadata"].building.address,
                            buildingName: featureOfInterest["metadata"].building.name
                        }).then(result => {
                        if (result && result.length > 0) {
                            // check if the room exits
                            dbContext.callExtension("runQuery",
                                `match (r:Room:Location {name: $roomName})-[:AT_LOCATION]->(f:Floor:Location {name: $floorName})
                                    -[:AT_LOCATION]->(b:Building:Location {name: $buildingName, address: $buildingAddress}) return r`, {
                                    floorName: featureOfInterest["metadata"].floor,
                                    buildingAddress: featureOfInterest["metadata"].building.address,
                                    buildingName: featureOfInterest["metadata"].building.name,
                                    roomName: featureOfInterest["metadata"].room
                                }).then(result => {
                                if (!(result && result.length > 0)) {
                                    // add thew new room
                                    dbContext.callExtension("runQuery",
                                        `match (b:Building:Location {name:$buildingName, address: $buildingAddress})<-[:AT_LOCATION]-
                                    (f:Floor:Location {name: $floorName})
                                    create (r:Room:Location {name: $roomName, prov:'Location'})-[:AT_LOCATION]->(f)`, {
                                            buildingAddress: featureOfInterest["metadata"].building.address,
                                            buildingName: featureOfInterest["metadata"].building.name,
                                            floorName: featureOfInterest["metadata"].floor,
                                            roomName: featureOfInterest["metadata"].room,
                                            featureOfInterestId: featureOfInterest["objectId"]
                                        }).then(result => {
                                        console.log("A new room added: ", featureOfInterest["metadata"]);
                                        FeatureOfInterestEventHandler.matchSensorsToRooms(dbContext, featureOfInterest);
                                    })
                                } else {
                                    FeatureOfInterestEventHandler.matchSensorsToRooms(dbContext, featureOfInterest);
                                }
                            })
                        } else {
                            // add the new floor and room
                            dbContext.callExtension("runQuery",
                                `match (b:Building:Location {name:$buildingName, address: $buildingAddress})
                            create (b)<-[:AT_LOCATION]-(f:Floor:Location {name: $floorName})
                            <-[:AT_LOCATION]-(r:Room:Location {name: $roomName})`, {
                                    buildingAddress: featureOfInterest["metadata"].building.address,
                                    buildingName: featureOfInterest["metadata"].building.name,
                                    floorName: featureOfInterest["metadata"].floor,
                                    roomName: featureOfInterest["metadata"].room,
                                    featureOfInterestId: featureOfInterest["objectId"]
                                }).then(result => {
                                console.log(`Floor and room inserted: `, featureOfInterest["metadata"]);
                                FeatureOfInterestEventHandler.matchSensorsToRooms(dbContext, featureOfInterest);
                            })
                        }
                    })
                } else {
                    // add all locations
                    dbContext.callExtension("runQuery",
                        `create (b:Building:Location {name:$buildingName, address: $buildingAddress})<-[:AT_LOCATION]-
                                    (f:Floor:Location {name: $floorName})<-[:AT_LOCATION]-
                                    (r:Room:Location {name: $roomName})`, {
                            buildingAddress: featureOfInterest["metadata"].building.address,
                            buildingName: featureOfInterest["metadata"].building.name,
                            floorName: featureOfInterest["metadata"].floor,
                            roomName: featureOfInterest["metadata"].room,
                            featureOfInterestId: featureOfInterest["objectId"]
                        }).then(result => {
                        console.log(`Location data inserted: `, featureOfInterest["metadata"]);
                        FeatureOfInterestEventHandler.matchSensorsToRooms(dbContext, featureOfInterest);
                    })
                }
            })
        } else {
            this.working = false;
        }
    }

    /**
     * Match sensors to their locations by creating relations on the graph database
     * @param dbContext
     * @param featureOfInterest
     * @param attempt
     */
    public static matchSensorsToRooms(dbContext: DbContext, featureOfInterest: {}, attempt: number = 0) {
        // do not attempt to match sensors to rooms after 1 second, 1*100 ms.
        if (attempt > 10) {
            FeatureOfInterestEventHandler.iterateLocationQueue(dbContext);
            return;
        }

        dbContext.callExtension("runQuery",
            `match (t:Thing {objectId: $parentObjectId}), (r:Room:Location {name: $roomName})-[:AT_LOCATION]->
                (f:Floor:Location {name: $floorName})-[:AT_LOCATION]->(b:Building:Location {address: $buildingAddress})
                merge (t)-[:AT_LOCATION]->(r) return t,r`, {
                parentObjectId: featureOfInterest["parentObjectId"],
                roomName: featureOfInterest["metadata"].room,
                floorName: featureOfInterest["metadata"].floor,
                buildingAddress: featureOfInterest["metadata"].building.address
            }).then(result => {
            if (!(result && result.length > 0)) {
                // check if the matching successful, schedule another method call in 100 ms in case it wasn't successful
                setTimeout(FeatureOfInterestEventHandler.matchSensorsToRooms, 100, dbContext, featureOfInterest, ++attempt);
            } else {
                FeatureOfInterestEventHandler.iterateLocationQueue(dbContext);
            }
        });
    }
}
