/**
* MIT License
*
* Copyright (c) 2020 fortiss GmbH
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
* 
* SPDX-License-Identifier: MIT
* @author Erkan Karabulut 
* @version 1.0
 * Includes Coaty sensor event handlers which interacts with the Neo4j DB
 * in order to query/insert/delete/update sensors, IoT things or groupings
 */

import {DbContext} from "@coaty/core/db";
import {JSONUtil} from "../util/JSONUtil";
import {ThingRepository} from "../repository/ThingRepository";
import {Events} from "../../service/Events";

export class SensorEventHandler {
    static working = false;
    static queue = [];

    /**
     * Runs when a new sensor is discovered
     * @param communicationManager
     * @param eventId
     * @param dbContext
     * @param sensor
     */
    public static sensorAddedEventHandler(communicationManager, eventId, dbContext: DbContext, sensor) {
        // process sensor discoveries one by one, by adding them in a queue
        this.queue.push(sensor);
        if (!this.working) {
            this.iterateSensorQueue(communicationManager, eventId, dbContext);
            return;
        }
    }

    /**
     * Iterate over the sensor queue and process each discovery separately
     * @param communicationManager
     * @param eventId
     * @param dbContext
     */
    public static iterateSensorQueue(communicationManager, eventId, dbContext: DbContext) {
        this.working = true;
        // get thing and sensor objects and linearize them
        let sensor = this.queue.pop();
        if (!sensor) {
            this.working = false;
            return;
        }

        let thingObject = JSONUtil.linearize(sensor.thing);
        let sensorObject = JSONUtil.linearize(sensor);

        // check if the IoT thing exists
        ThingRepository.getIoTThingByObjectId(dbContext, thingObject["objectId"]).then(thing => {
            if (thing && thing.length > 0) {
                // get sensors of the IoT thing
                ThingRepository.getSensorByParentObjectId(dbContext, thingObject["objectId"]).then(sensorList => {
                    for (let index in sensorList) {
                        // check if the new sensor exists
                        if (sensorObject["objectId"] == sensorList[index]._fields[0].properties.objectId) {
                            console.warn("IoT Thing and sensor already exists!");
                            this.iterateSensorQueue(communicationManager, eventId, dbContext);
                            return;
                        }
                    }

                    // add the new sensor
                    dbContext.callExtension("runQuery",
                        `with apoc.convert.fromJsonMap($sensor) as sensorObject
                        match (t:Thing {objectId: $thingObjectId})
                        create (s:Sensor)<-[:ATTACHED_TO]-(t)
                        set s=sensorObject
                        return s`, {
                            thingObjectId: thingObject["objectId"],
                            sensor: JSON.stringify(sensorObject)
                        })
                        .then(r => {
                            if (r == false) {
                                console.error("Failed to insert new sensor: ", sensorObject["objectId"]);
                            } else {
                                console.log("Added a new sensor: ", sensorObject["name"] + " - " + sensorObject["objectId"])
                            }
                            this.iterateSensorQueue(communicationManager, eventId, dbContext);
                        })
                });
            } else {
                // insert IoT thing
                dbContext.callExtension("runQuery",
                    `with apoc.convert.fromJsonMap($thing) as thingObject,
                    apoc.convert.fromJsonMap($sensor) as sensorObject
                    create (s:Sensor)<-[:ATTACHED_TO]-(t:Thing)
                    set s=sensorObject
                    set t=thingObject
                    return t`, {
                        thing: JSON.stringify(thingObject),
                        sensor: JSON.stringify(sensorObject)
                    })
                    .then(r => {
                        if (r == false) {
                            console.error("Failed to insert IoT Thing: ", thingObject["objectId"]);
                        } else {
                            console.log("Added a new IoT Environment thing: ", thingObject["name"] + " - " + thingObject["objectId"])
                            console.log("Added a new sensor: ", sensorObject["name"] + " - " + sensorObject["objectId"]);
                        }
                        this.iterateSensorQueue(communicationManager, eventId, dbContext);
                    })
            }
        });
    }

    /**
     * Runs when a sensor is deadvertised and deletes the sensor from db
     * @param communicationManager
     * @param eventId
     * @param dbContext
     * @param sensor
     */
    public static sensorDeletedEventHandler(communicationManager, eventId, dbContext: DbContext, sensor) {
        if (this.working == true) {
            setTimeout(() => {
                this.sensorDeletedEventHandler(communicationManager, eventId, dbContext, sensor);
            }, 100);
            return;
        }

        this.working = true;
        let sensorObject = JSONUtil.linearize(sensor);
        let thingObject = JSONUtil.linearize(sensor.thing);

        // delete the sensor from db
        ThingRepository.deleteSensorByObjectId(dbContext, sensorObject["objectId"]).then(result => {
            console.log("Sensor is deleted: ", sensorObject["name"], " - ", sensorObject["objectId"]);
            ThingRepository.getSensorByParentObjectId(dbContext, sensorObject["parentObjectId"]).then(sensorList => {
                if (sensorList.length == 0) {
                    // check if are there any other sensor attached to the thing, if not delete the thing as well
                    ThingRepository.deleteThingByObjectId(dbContext, thingObject["objectId"]).then(result => {
                        console.log("IoT Thing is deleted: ", thingObject["name"], " - ", thingObject["objectId"]);
                        this.working = false;
                        Events._advertiseDatabaseChange(communicationManager, eventId);
                    });
                } else {
                    this.working = false;
                }
            })
        });
    }
}
