/**
* MIT License
*
* Copyright (c) 2020 fortiss GmbH
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
* 
* SPDX-License-Identifier: MIT
* @author Erkan Karabulut 
* @version 1.0
 * DB operations related with sensor-service request grouping
 */

import {DbContext} from "@coaty/core/db";

/**
 * DB operations related with the sensor and service request groupings
 */
export class GroupingRepository {
    /**
     * Get service requests that are not yet processed by the tsmatch engine
     * @param dbContext
     */
    public static getNonProcessedGroupings(dbContext: DbContext) {
        return dbContext.callExtension("runQuery",
            `match (r:Request {isProcessed: false})<-[rel:USED_BY]-(s:Sensor) return r,s,rel`);
    }

    /**
     * Get all sensor service request groupings
     * @param dbContext
     */
    public static getAllGroupings(dbContext: DbContext) {
        return dbContext.callExtension("runQuery",
            `match (r:Request)<-[rel:USED_BY]-(s:Sensor)<-[:ATTACHED_TO]-(t:Thing)-[:AT_LOCATION]->
            (r2:Room)-[:AT_LOCATION]->(f:Floor)-[:AT_LOCATION]->(b:Building) return r,s,rel,r2.name,f.name,b.name,b.address`);
    }

    /**
     * Get all the groupings that are processed by the tsmatch engine but a response isn't sent to the user for that grouping
     * @param dbContext
     */
    public static getNotSentSuccessfulGroupings(dbContext: DbContext) {
        return dbContext.callExtension("runQuery",
            `match (r:Request {responseSent: false, isProcessed: true})<-[rel:USED_BY]-(s:Sensor)<-[:ATTACHED_TO]-(t:Thing)-[:AT_LOCATION]
            ->(room:Room)-[:AT_LOCATION]->(f:Floor)-[:AT_LOCATION]->(b:Building) return r,s,rel,room,f,b`);
    }

    /**
     * Get all the unsuccessful groupings that are not yet sent to the user
     * @param dbContext
     */
    public static getNotSentUnsuccessfulGroupings(dbContext: DbContext) {
        return dbContext.callExtension("runQuery",
            `match (r:Request {responseSent: false, isProcessed: true})
            where not (r)<-[:USED_BY]-(:Sensor)
            return r`);
    }

    /**
     * Mark a given grouping as sent to the user
     * @param dbContext
     * @param requestObjectId
     */
    public static markGroupingAsResponseSent(dbContext: DbContext, requestObjectId: string) {
        return dbContext.callExtension("runQuery",
            `match (r:Request {objectId: $requestObjectId})
            set r.responseSent=true`, {
                requestObjectId: requestObjectId
            });
    }
}
