/**
* MIT License
*
* Copyright (c) 2020 fortiss GmbH
*
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
* 
* SPDX-License-Identifier: MIT
* @author Erkan Karabulut 
* @version 1.0
 * DB operations related with location of the sensors/iot things
 */

import {DbContext} from "@coaty/core/db";

/**
 * Location related db operations
 */
export class LocationRepository {
    /**
     * get sensors and group by building
     * @param dbContext
     */
    static getSensorsByBuilding(dbContext: DbContext) {
        return dbContext.callExtension("runQuery",
            `match (s:Sensor)-[*4]-(b:Building)
            with b.address as buildingAddress, s as sensor, count(s) as sensorCount
            return buildingAddress, sensor`);
    }

    /**
     * get sensors and group by floor
     * @param dbContext
     */
    static getSensorsByFloor(dbContext: DbContext) {
        return dbContext.callExtension("runQuery",
            `match (s:Sensor)-[*3]-(f:Floor)-[:AT_LOCATION]->(b:Building)
            with b.address as buildingAddress, f.name as floorName, s as sensor, count(s) as sensorCount
            return buildingAddress, floorName, sensor`);
    }

    /**
     * get sensors and group by room
     * @param dbContext
     */
    static getSensorsByRoom(dbContext: DbContext) {
        return dbContext.callExtension("runQuery",
            `match (s:Sensor)-[*2]-(r:Room)-[:AT_LOCATION]->(f:Floor)-[:AT_LOCATION]->(b:Building)
            with b.address as buildingAddress, f.name as floorName, r.name as roomName, s as sensor, count(s) as sensorCount
            return buildingAddress, floorName, roomName, sensor`);
    }

    /**
     * get sensors and group by iot thing
     * @param dbContext
     */
    static getSensorsByDevice(dbContext: DbContext) {
        return dbContext.callExtension("runQuery",
            `match (s:Sensor)<-[:ATTACHED_TO]-(t:Thing)-[:AT_LOCATION]->(r:Room)-[:AT_LOCATION]->(f:Floor)-[:AT_LOCATION]->(b:Building)
            with b.address as buildingAddress, f.name as floorName, r.name as roomName, s as sensor, t.objectId as thing, count(s) as sensorCount
            return buildingAddress, floorName, roomName, thing, sensor `);
    }



    /**
     * Get featureOfInterest(location) of a given iot thing
     * @param dbContext
     * @param thingObjectId
     */
    static getFeatureOfInterest(dbContext: DbContext, thingObjectId: string) {
        return dbContext.callExtension("runQuery",
            `match (t:Thing {objectId: $thingObjectId})
            -[:AT_LOCATION]->(r:Room)-[:AT_LOCATION]->(f:Floor)-[:AT_LOCATION]->(b:Building)
            return r.name, f.name, b.name, b.address`, {
                thingObjectId: thingObjectId
            });
    }

    /**
     * get a list of all location nodes
     * @param dbContext
     */
    static getAllLocationNodes(dbContext: DbContext) {
        return dbContext.callExtension("runQuery",
            `match (r:Room)-[:AT_LOCATION]->(f:Floor)-[:AT_LOCATION]->(b:Building) return r,f,b`);
    }
}
